#!/usr/bin/env python
import os
import sys

if True: 
    #~ don't copy this part when following the tutorial
    from os.path import dirname, join, abspath
    srcdir = abspath(join(dirname(__file__),'..'))
    if not srcdir in sys.path:
        sys.path.append(srcdir)
    #~ print sys.path
    
os.environ['DJANGO_SETTINGS_MODULE'] = 't1.settings'

import settings

from django.core.management import execute_manager

execute_manager(settings)










# -*- coding: UTF-8 -*-
## Copyright 2011-2012 Luc Saffre
## This file is part of the Lino project.
## Lino is free software; you can redistribute it and/or modify 
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## Lino is distributed in the hope that it will be useful, 
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with Lino; if not, see <http://www.gnu.org/licenses/>.

import os
import sys

from lino.modlib.vocbook.fr import French, Autre, Nom, NomPropre, Adjectif, Numerique, Verbe, NomGeographique
from lino.modlib.vocbook.et import Estonian
from lino.modlib.vocbook.base import Book, FR, M, F, ET, PRON, GEON, GEOM, GEOF

if len(sys.argv) != 3:
    raise Exception("""
Usage : %(cmd)s rst OUTPUT_ROOT_DIR
    %(cmd)s odt OUTPUT_FILE
""" % dict(cmd=sys.argv[0]))
fmt = sys.argv[1]
if fmt == "rst":
    FULL_CONTENT = True
else:
    FULL_CONTENT = False

book = Book(French,Estonian,
    title=u"Kutsealane prantsuse keel kokkadele",
    input_template=os.path.join(os.path.dirname(__file__),'Default.odt'))
    #~ os.path.join(os.path.dirname(__file__),'cfr.odt')

Pronounciation = book.add_section(u"Hääldamine",intro=u"""
Esimeses osas keskendume hääldamisele.
Siin pole vaja meelde jätta näidissõnu,
vaid et sa oskaksid neid ette lugeda õigesti hääldades.
""")
Intro = Pronounciation.add_section(u"Kirjeldus",intro=u"""
""")
Reeglid = Pronounciation.add_section(u"Reeglid",ref="reeglid")
if FULL_CONTENT:
    Eesti = Pronounciation.add_section(u"Veel")

#~ Vocabulary = book.add_section(u"Sõnavara",intro=u"""
#~ Teises osa hakkame õpima sõnavara,
#~ oletades et hääldamine on enam vähem selge.
#~ """)
Vocabulary = book
#~ General = Vocabulary.add_section(u"Üldiselt")
General = Vocabulary.add_section(u"Üldine sõnavara")
Kokadele = Vocabulary.add_section(u"Kulinaaria")
if FULL_CONTENT:
  Fun = Vocabulary.add_section(u"Fun")


Intro.add_lesson(u"Tuntud sõnad", intro=u"""
Mõned sõnad, mida sa juba tead.
Tutvumine hääldamiskirjaga.
""")
Intro.parse_words(None,u"""
la soupe [sup] : supp
la carte [kart] : kaart
à la carte [ala'kart] : menüü järgi
le vase [vaaz] : vaas
la douche [duš] : dušš
le disque [disk] : ketas
merci [mär'si] : aitäh
le garage [ga'raaž] : garaaž
le journal [žur'nal] : päevik | ajaleht
""")
Intro.add_after(u"""
Kuna hääldamine on algaja peamine raskus, 
tuleb meil seda kuidagi kirja panna.
Seda teeme sõnade taha nurksulgudes (**[]**).

- **ou** hääldatakse **[u]**.

- **e** sõna lõpus kaob ära

- Prantsuse keeles on raske ette teada, kas täishäälikuid 
  hääldatakse lühikesena või pikana.
  Hääldamiskirjelduses kirjutame pikad 
  kaashäälikud topelt.

- Prantsuse keeles on rõhk tavaliselt viimasel silbil.
  Hääldamiskirjelduses paneme apostroofi (') rõhutatud silbi ette.
 
""")


Intro.add_lesson(u"Hääldamiskirjeldus", intro=u"""
Hääldamiskirjeldustes kasutame 
kohandatud `X-SAMPA 
<http://fr.wiktionary.org/wiki/Annexe:Prononciation/fran%C3%A7ais>`_ variant, 
mis on eestlastele intuitiivsem õppida 
kui näiteks `IPA 
<http://en.wiktionary.org/wiki/Wiktionary:IPA>`_ (International 
Phonetic Alphabet).

Üldiselt loed lihtsalt seda, mis on nurksulgudes.
Välja arvatud mõned helid, mida tuleb õppida:

==== ================== =================== =======================================
täht selgitus           näided e.k.         näided pr.k.
==== ================== =================== =======================================
[ə]  tumm e             Lott\ **e**         **je** [žə], **ne** [nə]
[o]  kinnine o          L\ **oo**\ ne       **mot** [mo], **beau** [boo]
[O]  avatud o           L\ **o**\ tte       **bonne** [bOn], **mort** [mOOr]
[ö]  kinnine ö          l\ **öö**\ ve       **feu** [föö], **peu** [pöö]
[Ö]  avatud ö           ingl. "g\ *ir*\ l"  **beurre** [bÖÖr], **jeune** [žÖÖn]
[w]  ingl. k. "where"   ingl. "\ **w**\ e"  **trois** [trw'a], **foie** [fw'a]
[O~] nasaalne [O]       -                   **bonjour** [bO~'žuur], **mon** [mO~]
[A~] nasaalne [A]       -                   **tante** ['tA~tə], **prendre** ['prA~drə]
[Ö~] nasaalne [Ö]       -                   **un** [Ö~], **parfum** [par'fÖ~]
[Ä~] nasaalne Ä         -                   **chien** [šiÄ~], **rien** [riÄ~]
==== ================== =================== =======================================

Heli **[ö]** on eesti keeles alati *kinnine* (hääldamiskirjelduses väikese tähega). 
Prantsuse keeles on lisaks *avatud* **[ÖÖ]** (hääldamiskirjelduses suure tähega).
Ja nende erinevus on oluline.
Hääldamiskirjelduses on *kinnine* **ö** *väikese* tähega, 
*avatud* **Ö** *suure* tähega.

Heli **[w]** on "ülipehme", 
nagu nt. inglise sõnades "week", "wow", "where".

""")

Intro.add_lesson(u"Mesilashäälikud", intro=u"""
"Mesilashäälikud" on **s**, **š**, **z** ja **ž**.
  
=========== ===========================
terav       pehme
=========== ===========================
**s**\ upp  **z**\ oom
**š**\ okk  **ž**\ urnaal, **ž**\ anre
=========== ===========================
  
Nad on eesti keeles ka olemas, aga prantsuse keeles on
nende erinevus palju olulisem.
""")
Intro.parse_words(None,u"""
la soupe [sup] : supp
le garage [ga'raaž] : garaaž
le genre [žA~rə] : žanre
le journal [žur'nal] : päevik | ajaleht
le choc [žok] : šokk | löök
""")




Intro.add_lesson(u"Artikkel", intro=u"""
Prantsuse keeles on kõikidel asjadel oma sugu.
Näiteks laud (*la table*) on naissoost, 
raamat (*le livre*) on meessoost.

Kui sul on mitu lauda või mitu raamatu, 
siis on neil sama artikkel **les** (nt. *les tables*, *les livres*).

Kui sõna algab täishäälikuga, siis kaob 
artiklitest *le* ja *la* viimane 
täht ära ja nad muutuvad  mõlemad **l'**-ks.

Artiklid *le*, *la* ja *les* nimetatakse *määravaks* artikliteks.

Määrava artikli asemel võib ka olla **umbmäärane** artikkel: **un** (meessoost) 
või **une** (naissoost).
Erinevus on nagu inglise keeles, kus on olemas määrav 
artikkel **the** ja umbmäärane artikel **a**.
Olenevalt kontekstist kasutatakse kas see või teine.
Näiteks
"I am **a** man from Vigala"
ja 
"I am **the** man you need".

Mitmuses on umbmäärane artikkel **des** sama mõlemas soos.

Nii et kokkuvõteks:

========== ============= =============
sugu       määrav        umbmäärane
========== ============= =============
meessoost  **le** [lə]   **un** [Ö~]
naissoost  **la** [la]   **une** [ün]
mitmus     **les** [lä]  **des** [dä]
========== ============= =============

""")
#~ Intro.parse_words(Autre,u"""
#~ le [lə] : (määrav meessoost artikkel)
#~ la [la] : (määrav naissoost artikkel)
#~ les [lä] : (määrav artikkel mitmus)
#~ """)
#~ Intro.parse_words(Autre,u"""
#~ un [Ö~] : (umbmäärane meessoost artikkel)
#~ une [ün] : (umbmäärane naissoost artikkel)
#~ des [dä] : (umbmäärane artikkel mitmus)
#~ """)

Intro.add_lesson(u"Rõhutud, aga lühike!", intro=u"""
Rõhutatud täishäälikud ei ole sellepärast tingimata pikad.
Prantsuse keeles tuleb tihti ette, et sõna lõpeb *lühikese* täishäälikuga.
""")
Intro.parse_words(Nom,u"""
le menu [mə'nü] : menüü
le chocolat [šoko’la] : šokolaad
le plat [pla] : roog | kauss
le cinéma [sine’ma] : kino
le paradis [para'di] : paradiis
""")


Reeglid.add_lesson(u"u", intro=u"""
**u** (siis kui see pole teise täishäälikuga koos)
hääldatakse **[ü]** või **[üü]**.
""",ref="u")
Reeglid.parse_words(Nom,u"""
le bureau [bü'roo] : büroo
le bus [büs] : buss
le mur [müür] : sein | müür
la puce [püs] : kirp
le jus [žü] : mahl
le but [büt] : eesmärk
la pute [püt] : hoor
le sucre ['sükrə] : suhkur
""")

Reeglid.add_lesson(u"ou", intro=u"""
**ou** hääldatakse **[u]** või **[uu]**.
""",ref="ou")
Reeglid.parse_words(None,u"""
le cours [kuur] : kursus | tund (koolis)
le cou [ku] : kael
le goût [gu] : maitse
le chou [šu] : kapsas
le loup [lu] : hunt
un ours [urs] : karu
le journal [žur'nal] : päevik
""")

Reeglid.parse_words(None,u"""
la loupe [lup] : luup
la joue [žuu] : põsk
le jour [žuur] : päev
court (m) [kuur] : lühike
mou (m) [mu] : pehme
""")
#~ Reeglid.parse_words(NomPropre,u"""
#~ Winnetou [winə'tu] : (isegi maailmakuulsa apatši pealiku nime hääldavad prantslased valesti)
#~ """)


Reeglid.add_lesson(u"oi", 
u"""
**oi** hääldatakse **[wa]**.
Vaata ka [ref oin].
""",ref="oi")
Reeglid.parse_words(Autre,u"""
voilà [vwa'la] : näe siin 
trois [trwa] : kolm
bonsoir [bO~'swaar] : head õhtut
au revoir [orə'vwaar] : nägemiseni
""")
Reeglid.parse_words(Nom,u"""
le roi [rwa] : kuningas
la loi [lwa] : seadus
la toilette [twa'lät] : tualett
un oiseau [wa'zoo] : lind
""")



Reeglid.add_lesson(u"*au* ja *eau*", 
intro=u"""
**au** hääldatakse **[o]** või **[oo]**.
Kui **e** on veel ees, siis see sulab ka nendega kokku.
""",ref="au")
Reeglid.parse_words(None,u"""
une auberge [o'bäržə] : võõrastemaja
un auteur [o'tÖÖr] : autor
le château [ža'too] : loss
le bateau [ba'too] : laev
la eau [oo] : vesi
""")



Reeglid.add_lesson(u"y", u"""
**y** hääldatakse alati **[i]** ja mitte kunagi **[ü]**.
""",ref="y")
Reeglid.parse_words(Nom,u"""
le cygne ['sinjə] : luik
le système [sis'tääm] : süsteem
le mythe [mit] : müüt
""")


Reeglid.add_lesson(u"ai", 
u"""
**ai** hääldatakse **[ä]** või **[ää]** 
(mõnikord ka **[ə]**).
""",ref="ai")
Reeglid.parse_words(Nom,u"""
la maison [mä'zO~] : maja
le domaine [do'mään] : domeen
la fraise [frääz] : maasikas
la paire [päär] : paar
""")
Reeglid.parse_words(Adjectif,u"""
frais [frä] | fraiche [fräš] : värske
""")
Reeglid.parse_words(None,u"""
nous [nu fə'zO~] : meie teeme
le faisan [fə'zA~] : faasan
""")


Reeglid.add_lesson(u"ail", 
u"""
**ail** hääldatakse **[aj]**
(siin ei kehti reegel [ref ai], sest *i* sulab *l*-iga kokku.)
""",ref="ail")
Reeglid.parse_words(Nom,u"""
l'ail (m) [aj] : küüslauk
le travail [tra'vaj] : töö
le détail [detaj] : detail
l'aile (f) [ääl] : tiib
""")
Reeglid.parse_words(NomGeographique,u"""
Versailles [ver'sajə] : Versailles
""")





Reeglid.add_lesson(u"j", 
u"""
**j** hääldatakse **[ž]** (ja mitte [dž]).
""",ref="j")
Reeglid.parse_words(None,u"""
majeur [mažÖÖr] : suurem
je [žə] : mina
jamais [ža'mä] : mitte iialgi
jaune (mf) [žoon] : kollane
""")
Reeglid.parse_words(NomPropre,u"""
Josephe [žo'zäf] : Joosep
""")


Reeglid.add_lesson(u"g", 
u"""
**g** hääldatakse **[g]** kui järgneb **a**, **o**, **u** 
või kaashäälik, aga **[ž]** kui järgneb **e**, **i** või **y**.
""",ref="g")
Reeglid.parse_words(None,u"""
le gorille [go'rijə] : gorilla
la gazelle [ga'zäl] : gasell
la giraffe [ži'raf] : kaelkirjak
le gymnase [žim'naaz] : gümnaasium
le juge [žüüž] : kohtunik
la géologie [žeolo'žii] : geoloogia
général [žene'ral] : üldine
""")

Reeglid.add_lesson(u"gu", 
u"""
**gu** hääldatakse **[g]** (s.t. **u** kaob ära) 
siis kui järgneb **e**, **i** või **y**.
Muidu on hääldamine **[gu]**
""",ref="gu")
Reeglid.parse_words(None,u"""
le guépard [ge'paar] : gepard
le guide [giid] : reisijuht
la guitare [gi'taar] : kitarr
la guerre [gäär] : sõda
Guy [gi] : (eesnimi)
Gustave [güs'taav] : (eesnimi)
aigu [ä'gü] : terav, ...
""")






Reeglid.add_lesson(u"c", u"""
**c** hääldatakse **[s]** siis 
kui järgneb **e**, **i** või **y**,
ja muidu **[k]** (ja mitte kunagi **[tš]**).
""",ref="c")
Reeglid.parse_words(None,u"""
le câble ['kaablə] : kaabel
la comédie [kome'dii] : komöödia
le comble ['kO~blə] : kõrgeim v. ülim aste
la cure [küür] : kuur
la cible ['siiblə] : märklaud
le certificat [särtifi'ka] : tsertifikaat
la cire [siir] : vaha
le centre ['sA~trə] : keskus
le cygne ['sinjə] : luik
la classe [klas] : klass
la croûte [krut] : koorik
un acacia [akasj'a] : akaatsia (põõsas)
""")
Reeglid.parse_words(NomPropre,u"""
octobre [ok'tOObrə] : oktoober
Marc [mark] : Markus
""")
Reeglid.parse_words(Numerique,u"""
cinq [sÄ~k] : viis
""")

Reeglid.add_lesson(u"ç", u"""
**ç** hääldatakse alati **[s]**.
""",ref="cedille")
Reeglid.parse_words(None,u"""
la leçon [lə~sO~]: lektsioon
la rançon [rA~sO~]: lunaraha
le reçu [rə'sü] : kviitung
le maçon [ma'sO~] : müürsepp
""")


#~ Pronounciation.parse_words(Autre,u"""
#~ Il était une fois [iletätünə'fwa] : Oli üks kord
#~ le boudoir [bud'waar] : buduaar
#~ """)

#~ Intro.add_lesson(u"Jah ja ei", u"""
#~ """)
#~ Intro.parse_words(None,u"""
#~ """)


Reeglid.add_lesson(u"ui", 
u"""
**ui** hääldatakse **[wi]** või **[wii]** (mida 
kirjutatakse vahest ka **[üi]** või **[üii]**).
""",ref="ui")
Reeglid.parse_words(None,u"""
la suite [swit] : järg | tagajärg | rida, kord | saatjaskond
bonne nuit [bOn'nwi] : head ööd
la cuisine [kwi'zin] : köök
je cuis [žə kwi] : ma keedan
je suis [žə swi] : ma olen | ma järgnen
""")

#~ Reeglid.parse_words(None,u"""
#~ l'huile (f) [wil] : õli
#~ cuire [kwiir] : keetma
#~ suivre ['swiivrə] : järgima
#~ la cuillère [kwi'jäär] : lusikas
#~ """)


Reeglid.add_lesson(u"un, um", 
u"""
**um** ja **un** hääldatakse **[Ö~]**,
v.a. siis kui järgneb täishäälik või teine **m** / **n**.
""",ref="un")
Reeglid.parse_words(Nom,u"""
le parfum [par'fÖ~] : hea lõhn v. maitse
""")
Reeglid.parse_words(NomPropre,u"""
Verdun [vär'dÖ~] : -
""")
Reeglid.parse_words(Adjectif,u"""
brun [brÖ~] | brune [brün] : pruun
aucun [o'kÖ~] | aucune [o'kün] : mitte üks
parfumé [parfü'mee] | parfumée [parfü'mee] : lõhnastatud
""")

#~ chacun [ža'kÖ~] | chacun [ža'kün] : igaüks



Intro.add_lesson(u"Lotte ja Loone", u"""
Hääldamiskirjelduses on olemas *kinnine* [o] (väikese tähega)
ja *avatud* [O] (suure tähega).

Lotte **O** on lühike ja *avatud*, Loone oma on pikk ja *kinnine*.
Prantsuse keeles on lisaks veel olemas *avatud ja pikk* [OO].
""")
Intro.parse_words(Autre,u"""
je donne [dOn] : ma annan
je dors [dOOr] : ma magan
""")
Intro.parse_words(Nom,u"""
la mort [mOOr] : surm
le mot [mo] : sõna
le or [OOr] : kuld
le boulot [bu'lo] : töö (kõnekeel)
le bouleau [bu'loo] : kask
le bureau [bü'roo] : büroo
""")

if FULL_CONTENT:

    Fun.add_lesson(u"Frère Jacques", 
    u"""
    | Frère Jacques, frère Jacques,
    | dormez-vous? Dormez-vous? 
    | Sonnez les matines, sonnez les matines
    | ding, dang, dong! Ding, dang, dong!
    """)
    Fun.parse_words(NomPropre,u"""
    Jacques [žaak] : Jaak
    """)
    Fun.parse_words(None,u"""
    le frère [fräär] : vend
    dormez-vous? [dOrme'vu] : kas Te magate?
    Sonnez les matines [sO'ne lä ma'tinə] : lööge hommikukellad
    """)

if FULL_CONTENT:

    Fun.add_lesson(u"Minu onu...", 
    u"""
    | Mon tonton et ton tonton sont deux tontons,
    | mon tonton tond ton tonton 
    | et ton tonton tond mon tonton.
    | Qu'est-ce qui reste?

    """)
    Fun.parse_words(None,u"""
    mon [mO~] : minu
    ton [tO~]: sinu
    ils sont [sO~]: nad on
    """)
    Fun.parse_words(Numerique,u"""
    deux [döö] : kaks
    """)
    Fun.parse_words(Nom,u"""
    un tonton [tO~'tO~] : onu
    """)
    Fun.parse_words(Verbe,u"""
    tondre [tO~drə] : pügama
    rester [räs'tee] : üle jääma
    """)
    Fun.parse_words(None,u"""
    Qu'est-ce qui reste? [käski'räst?] : Mis jääb üle?
    """)


#~ """
#~ le nôtre ['nootrə] : meie oma
#~ """

Reeglid.add_lesson(u"eu", u"""
**eu** hääldatakse **[öö]** või **[ÖÖ]**.
""",ref="eu")
Reeglid.parse_words(Nom,u"""
le feu [föö] : tuli
le neveu [nə'vöö] : onupoeg | tädipoeg
je veux [žə vöö] : ma tahan
""")
Reeglid.parse_words(Autre,u"""
neutre (mf) ['nöötrə] : neutraalne
""")
Reeglid.parse_words(Numerique,u"""
neuf [nÖf] : üheksa
""")
Reeglid.parse_words(Nom,u"""
le professeur [profesÖÖr] : professor
le beurre [bÖÖr] : või
la peur [pÖÖr] : hirm
""")

Reeglid.add_lesson(u"œ", u"""
""")
Reeglid.parse_words(Nom,u"""
le nœud [nöö] : sõlm
le cœur [kÖÖr] : süda
le chœur [kÖÖr] : koor (laulu-)
le bœuf [bÖff] : härg
un œuf [Öf] : muna
une œuvre [ÖÖvrə] : töö, teos
le *hors d'œuvre [hOOr 'dÖÖvrə] : eelroog
""")





Reeglid.add_lesson(u"on ja om", 
u"""
**on** ja **om** hääldatakse **[O~]**,
välja arvatud siis kui järgneb täishäälik või teine **n** või **m**.
""",ref="on")
Reeglid.parse_words(Nom,u"""
le salon [sa’lO~] : salong (= uhke tuba)
un oncle [O~klə] : onu
la bombe ['bO~mbə] : pomm
""")
Reeglid.parse_words(Autre,u"""
bonjour [bO~'žuur] : tere | head päeva | tere hommikust
bonne nuit [bOn'nwi] : head ööd
bon appétit [bOnappe'ti] : head isu
""")



Reeglid.add_lesson(u"an, am", 
u"""
**an** ja **am** hääldatakse **[A~]**,
välja arvatud siis kui järgneb täishäälik või teine **n** või **m**.
""",ref="an")
Reeglid.parse_words(Nom,u"""
un an [A~] : aasta
une année [a'nee] : aasta
la lampe [lA~pə] : lamp
un enfant [A~'fA~] : laps
""")

Reeglid.add_lesson(u"en, em", 
u"""
**en** ja **em** hääldatakse **[A~]**,
välja arvatud siis kui järgneb täishäälik või teine **n** või **m**.
""",ref="en")
Reeglid.parse_words(Nom,u"""
le rendez-vous [rA~de’vu] : kohtumine
le commentaire [komA~’täär] : märkus, kommentar
le centre ['sA~trə] : keskus
le genre [žA~rə] : žanre
un enfant [A~'fA~] : laps
un employeur [A~plwa'jÖÖr] : tööandja
""")

Reeglid.add_lesson(u"-ent", 
u"""
**ent** sõna lõpus hääldatakse **[ə]** siis kui tegemist 
on *tegusõna kolmada mitmuse vormiga*. 
Muidu kehtib reegel [ref en] (hääldatakse **[A~]**).
""",ref="ent")
Reeglid.parse_words(None,u"""
ils couvent [il 'kuuvə] : nad munevad
le couvent [ku'vA~] : klooster
souvent [su'vA~] : tihti
""")

Reeglid.add_lesson(u"in, im", 
u"""
**in** ja **im** hääldatakse **[Ä~]**,
välja arvatud siis kui järgneb täishäälik või teine **n** või **m**.
Vaata ka [ref ain].
""",ref="in")
Reeglid.parse_words(Nom,u"""
une information [Ä~formasjO~] : informatsioon
un imperméable [Ä~pärme'aablə] : vijmajope
une image [i'maaž] : pilt
le bassin [ba'sÄ~] : bassein
le dessin [de'sÄ~] : joonistus
le vin [vÄ~]: vein
""")
Reeglid.parse_words(Adjectif,u"""
inutile (mf) [inü'til] : kasutu
""")


Reeglid.add_lesson(u"ain, aim, ein, eim", 
u"""
Kui **a** või **e** on **in**/**im** ees, 
siis see sulab nendega kokku ja kaob ära.
""",ref="ain")
Reeglid.parse_words(Nom,u"""
le pain [pÄ~] : sai | leib
le gain [gÄ~] : kasu
la main [mÄ~] : käsi
la faim [fÄ~] : nälg
""")
Reeglid.parse_words(NomPropre,u"""
Reims [rÄ~s] : (linn)
""")

Reeglid.add_lesson(u"ien", 
u"""
**ien** hääldatakse **[jÄ~]** v.a. siis kui järgneb teine **n**.
""",ref="ien")
Reeglid.parse_words(None,u"""
le chien [šiÄ~] : koer
la chienne [šjän] : emakoer
""")
Reeglid.parse_words(Autre,u"""
bien [biÄ~] : hästi
rien [riÄ~] : ei midagi
""")

Reeglid.add_lesson(u"oin", 
u"""
**oin** hääldatakse **[wÄ~]**.
Reegel [ref oi] ei kehti sel juhul, sest *i* sulab *n*-iga kokku.
""",ref="oin")
Reeglid.parse_words(None,u"""
le coin [kwÄ~] : nurk
le point [pwÄ~] : punkt
""")
Reeglid.parse_words(Autre,u"""
besoin [bə'zwÄ~] : vaja
loin [lwÄ~] : kauge
""")



Reeglid.add_lesson(u'il', 
u"""
**il** (sõna lõpus ja kaashääliku taga)
hääldatakse kas **[i]** või **[il]**.
""",ref="il")
Reeglid.parse_words(None,u"""
il [il] : tema
le persil [pär'sil] : petersell
subtil (m) [süp'til] : peen, subtiilne
un outil [u'ti] : tööriist
le fusil [fü'zi] : püss
gentil (m) [žA~'ti] : armas
un exil [äg'zil] : eksiil
""")

Reeglid.add_lesson(u'*eil* ja *eille*', 
u"""
**eil** hääldatakse **[eij]**.
""",ref="eil")
Reeglid.parse_words(None,u"""
le réveil [re'veij] : äratuskell
le soleil [so'leij] : päike
la merveille [mär'veijə] : ime
merveilleux [märvei'jöö] : imeline
le réveillon [revei'jO~] : vana-aasta õhtu söök
la groseille [gro'zeijə] : sõstar (punane v. valge) | tikker
vieille (f) [vjeijə] : vana
# la veille [veijə] : pühalaupäev
""")
Reeglid.add_lesson(u"ueil", 
u"""
**ueil** hääldatakse **[Öj]**.
""",ref="ueil")
Reeglid.parse_words(None,u"""
un accueil [a'kÖj] : vastuvõtt
le chevreuil [šəv'rÖj] : metskits
un écureuil [ekü'rÖj] : orav
le orgueil [Or'gÖj] : ülbus
""")

Reeglid.add_lesson(u"ill", u"""
**ill** hääldatakse **[iij]** või  **[ij]**.
Erandid on sõnad *ville* ja *mille*.
""",ref="ill")
Reeglid.parse_words(None,u"""
la bille [biije] : kuul
une anguille [A~'giije] : angerjas
la myrtille [mir'tiije] : mustikas
la famille [fa'miije] : perekond
tranquille [trA~kiije] : rahulik
la cuillère [kwi'jäär] : lusikas
le pillage [pij'aaž] : rüüstamine
""")

Reeglid.parse_words(None,u"""
la ville [vil] : linn
mille [mil] : tuhat
le million [mil'jO~] : miljon
""")










Reeglid.add_lesson(u"h", u"""
**h** ei hääldata kunagi.
(Vaata ka [ref haspire])
""",ref="h")
Reeglid.parse_words(Nom,u"""
le hélicoptère [elikop'täär] : helikopter
le hôtel [o'täl] : hotell
le autel [o'täl] : altar
""")

if FULL_CONTENT:
  
  Reeglid.add_lesson(u"h aspiré", u"""
  Kuigi **h** ei hääldata kunagi ([vt. [ref h]]),
  on neid kaks tüüpi: «h muet» (tumm h) 
  ja «h aspiré» (sisse hingatud h).
  Viimane tähistatakse sõnaraamatutes tärniga (*).
  Erinevus koosneb selles, kuidas eesolev sõna liitub nendega.
  """,ref="haspire")
  Reeglid.parse_words(Nom,u"""
  le hélicoptère [elikop'täär] : helikopter
  le hôtel [o'täl] : hotell
  le homme [Om] : mees
  le *haricot [ari'ko] : uba
  le *héros [e'ro] : kangelane
  le *hibou [i'bu] : öökull
  """)





Reeglid.add_lesson(u"ch", u"""
**ch** hääldatakse tavaliselt **[š]** 
ja mõnikord (kreeka päritolu sõnades) **[k]**,
ja mitte kunagi **[tš]**.
""",ref="ch")
Reeglid.parse_words(Nom,u"""
le chat [ša] : kass
le chien [šjÄ~] : koer
la chèvre ['šäävrə] : kits
la biche [biš] : emahirv
un achat [a'ša] : ost
la chambre [šA~mbrə] : tuba
une chope [žOp] : õlu
le parachute [para'šüt] : langevari
le Christe [krist] : Kristus
le chœur [kÖÖr] : koor (laulu-)
le psychologue [psiko'lOOgə] : psüholoog
""")


Reeglid.add_lesson(u"gn", u"""
**gn** hääldatakse **[nj]**.
""",ref="gn")
Reeglid.parse_words(None,u"""
magnifique (nf) [manji'fik] : surepärane
le cognac [kon'jak] : konjak
la ligne ['linjə] : liin | rida
le signe ['sinjə] : märk
le signal [sin'jal] : signaal
la besogne [bə'zOnjə] : töö | tegu | ülesanne
""")
Reeglid.parse_words(Verbe,u"""
soigner [swan'jee] : ravima | hoolitsema
""")
Reeglid.parse_words(NomGeographique,u"""
Avignon [avin'jO~] : -
""")





if FULL_CONTENT:

    Eesti.add_lesson(u"b ja p", u"""
    b ja p on prantsuse keeles selgelt erinevad.
    """)
    Eesti.parse_words(None,u"""
    la bière [bjäär] : õlu
    la pierre [pjäär] : kivi
    le bon [bO~] : tšekk | talong
    le pont [pO~] : sild
    le bon ton [bO~'tO~] : viisakus
    le ponton [pO~'tO~] : pontoon (nt. pontoonsild)
    la peau [poo] : nahk
    beau (m.) : ilus
    le bois [bwa] : puu (materjal) | mets
    le poids [pwa] : kaal
    """)



    Eesti.add_lesson(u"d ja t", u"""
    d ja t on prantsuse keeles selgelt erinevad.
    """)
    Eesti.parse_words(None,u"""
    le don [dO~] : annetus
    le ton [tO~] : toon
    le centre ['sA~trə] : keskus
    la cendre ['sA~drə] : tuhk
    je donne [dOn] : ma annan
    la tonne [tOn] : tonn
    le toit [twa] : katus
    le doigt [dwa] : sõrm
    """)

    Eesti.add_lesson(u"g ja k", u"""
    g ja k on prantsuse keeles selgelt erinevad.
    """)
    Eesti.parse_words(None,u"""
    le gond [gO~] : uksehing
    le con [kO~] : loll
    la gare [gaar] : raudteejaam
    le car [kaar] : reisibuss
    car [kaar] : sest
    le garçon [gar'sO~] : poiss
    Qui est Guy? [ki ä gi] : Kes on Guy?
    """)



if False:
  
    Pronounciation.add_lesson(u"[äär]", u"""
    Kui kuuled [äär], siis kirjutad kas **ère**, **aire**, **ère**, **erre** või **er**.
    """)
    Pronounciation.parse_words(None,u"""
    le père [päär] : isa
    la paire [päär] : paar
    le maire [määr] : linnapea
    la mère [määr] : ema
    la mer [määr] : meri
    amer (m) [a'määr] : kibe
    la bière [bjäär] : õlu
    la pierre [pjäär] : kivi
    la terre [täär] : muld
    """)


if FULL_CONTENT:
  
    Eesti.add_lesson(u"v ja f", u"""
Ettevaatust, **v** ei ole **f**!
    """)
    Eesti.parse_words(None,u"""
    vous [vu] : teie
    fou [fu] : hull
    # vous êtes fous [vu'zäät fu] : te olete lollid
    je veux [žə vöö] : ma tahan
    le feu [föö] : tuli
    la fille [fiijə] : tüdruk | tütar
    la vie [vii] : elu
    la fin [fÄ~] : lõpp
    le vin [vÄ~] : vein
    """)

    Eesti.add_lesson(u"Ahv pole märk", u"""
Ettevaatust, **gn** ei ole **ng**!
    """)
    Eesti.parse_words(Nom,u"""
    un ange [A~ž] : ingel
    un agneau [an'joo] : tall
    le singe [sÄ~ž] : ahv
    le signe ['sinjə] : märk
    le linge [lÄ~ž] : pesu
    la ligne ['linjə] : liin | rida
    """)

    Eesti.add_lesson(u"Sugu on oluline", u"""
    Siin mõned näited, et sugu pole sugugi ebatähtis.
    """)
    Eesti.parse_words(Nom,u"""
    le père [päär] : isa
    la paire [päär] : paar
    le maire [määr] : linnapea
    la mère [määr] : ema
    le tour [tuur] : tiir
    la tour [tuur] : torn
    le mur [müür] : sein | müür
    la mûre [müür] : põldmari
    le cours [kuur] : kursus | tund (koolis)
    la cour [kuur] : õu, hoov | kohus
    """)


    Eesti.add_lesson(u"Ära aja segamini!", u"""
    Mõned ilusad harjutused veel.
    """)
    Eesti.parse_words(Autre,u"""
    ces ingrédients [säz Ä~gre'djA~] : need koostisained
    c'est un crétin [sätÖ~ kre’tÄ~] : ta on kretiin
    je dors [žə dOOr] : ma magan
    j'ai tort [žee tOOr] : ma eksin
    """)
    Eesti.parse_words(Nom,u"""
    la jambe [žA~mbə] : jalg
    la chambre [šA~mbrə] : tuba
    un agent [la' žA~] : agent
    le chant [lə šA~] : laul
    les gens [žA~] : inimesed, rahvas
    les chants [šA~] : laulud
    """)





General.add_lesson(u"Tervitused", u"""
""")
General.parse_words(Autre,u"""
salut [sa'lü] : tervist
bonjour [bO~'žuur] : tere | head päeva | tere hommikust
bonsoir [bO~'swaar] : head õhtut
bonne nuit [bOn'nwi] : head ööd
au revoir [orə'vwaar] : nägemiseni

Monsieur [məs'jöö] : härra
Madame [ma'dam] : proua
Mademoiselle [madəmwa'zel] : preili

Comment tu t'appelles? [ko'mA~ tü ta'päl] : Kuidas on sinu nimi?
Je m'appelle... [zə ma'päl] : Minu nimi on...
Comment vas-tu? [ko'mA~va'tü] : Kuidas sul läheb?

s'il vous plaît [silvu'plä] : palun (Teid)
s'il te plaît [siltə'plä] : palun (Sind)
merci [mer'si] : aitäh

oui [wi] : jah
non [nO~] : ei

bon appétit [bOnappe'ti] : head isu
j'ai faim [žee fÄ~] : mul on kõht tühi
j'ai soif [žee swaf] : mul on janu
je suis fatigué [žə swi fati'gee] : ma olen väsinud
""")


if FULL_CONTENT:
  
    General.add_lesson(u"Prantsuse automargid",columns=[FR,PRON],show_headers=False)
    General.parse_words(NomPropre,u"""
    Peugeot [pö'žo] : - 
    Citroën [sitro'än] : - 
    Renault [re'noo] : - 
    """)


    General.add_lesson(u"Prantsuse eesnimed", u"""
    """)
    General.parse_words(NomPropre,u"""
    Albert [al'bäär] : - 
    André [A~'dree] : Andre
    Anne [anə] : Anne
    Bernard [bär'naar] : - 
    Catherine [kat'rin] : Katrin
    Charles [šarl] : Karl
    François [frA~'swa] : -
    Isabelle [iza'bäl] : Isabel
    Jacques [žaak] : Jaak
    Jean [žA~] : Jaan
    Luc [lük] : Luukas
    Marie [ma'rii] : Maria
    Paul [pOl] : Paul
    Philippe [fi'lip] : Filip
    Pierre [pjäär] : Peeter
    """)

General.add_lesson(u"Taluloomad", u"""
""")
General.parse_words(Nom,u"""
la chèvre ['šäävrə] : kits
la brebis [brə'bis] : lammas
le porc [pOOr] : siga
le cochon [ko'šO~] : siga
le cheval [šə'val] : hobune
la vache [vaš] : lehm
le taureau [to'roo] : pull
le veau [voo] : vasikas
le bœuf [bÖff] : härg
""")

General.add_lesson(u"Metsloomad", u"""
""")
General.parse_words(Nom,u"""
la chasse [šas] : jaht
le chasseur [ša'sÖÖr] : jahimees
le chevreuil [šəv'rÖj] : metskits
le cerf [säär] : hirv
la biche [biš] : emahirv
un élan [e'lA~] : põder
le lapin [la'pÄ~] : küülik
le lièvre [li'äävrə] : jänes
le renard [rə'naar] : rebane
un écureuil [ekü'rÖj] : orav
la souris [su'ri] : hiir
le blaireau [blä'roo] : mäger | habemeajamispintsel
le *hérisson [eri'sO~] : siil
la hermine [är'min] : hermeliin
la martre ['martrə] : nugis
la belette [bə'lät] : nirk  
le loup [lu] : hunt
un ours [urs] : karu
le lynx [lÄ~ks] : ilves
le sanglier [sA~gli'e] : metssiga
le marcassin [marka'sÄ~] : metsseapõrsas
""")

# belette : nirk 

if FULL_CONTENT:

  Fun.add_lesson(u"Põdral maja", u"""
  | Dans sa maison un grand cerf 
  | regardait par la fenêtre
  | un lapin venir à lui         
  | et frapper ainsi.
  | «Cerf, cerf, ouvre moi
  | ou le chasseur me tuera!»    
  | «Lapin, lapin entre et viens 
  | me serrer la main.»          
  """)
  Fun.parse_words(Verbe,u"""
  il regardait [rəgar'dä] : ta vaatas
  ouvrir [uv'riir] : avama
  tuer [tü'ee] : tapma
  serrer [sä'ree] : suruma
  """)
  Fun.parse_words(Nom,u"""
  la maison [mä'zO~] : maja
  la fenêtre [fə'näätrə] : aken
  le cerf [säär] : hirv
  le lapin [lapÄ~] : küünik
  le chasseur [ša'sÖÖr] : jahimees
  la main [mÄ~] : käsi
  """)

General.add_lesson(u"Linnud", u"""
""")
General.parse_words(Nom,u"""
la poule [pul] : kana
le poulet [pu'lä] : tibu | kanapoeg
une oie [wa] : hani
le dindeon [dÄ~dO~] : kalkun
la dinde [dÄ~də] : emakalkun
le pigeon [pi'žO~] : tuvi
""")

General.add_lesson(u"Kalad", u"""
""")
General.parse_words(Nom,u"""
le brochet [bro'šä] : haug
une anguille [A~'giijə] : angerjas
la perche [pärš] : ahven
le *hareng [ar'~A] : heeringas
le sprat [sprat] : sprot
le thon [tO~] : tuunikala
le requin [rə'kÄ~] : haikala
""")






Kokadele.add_lesson(u"Katame lauda!", u"""
""")
Kokadele.parse_words(Nom,u"""
la table ['taablə] : laud
la chaise [šääz] : tool
le couteau [ku'too] : nuga
la fourchette [fur'šet] : kahvel
la cuillère [kwi'jäär] : lusikas
une assiette [as'jät] : taldrik
le verre [väär] : klaas
la tasse [tas] : tass
le plat [pla] : kauss
""")

Kokadele.add_lesson(u"Joogid", u"""""")
Kokadele.parse_words(Nom,u"""
la boisson [bwa'sO~] : jook
la bière [bjäär] : õlu
la eau [oo] : vesi
le jus [žu] : mahl
le café [ka'fee] : kohv
le thé [tee] : tee

le vin rouge [vÄ~ 'ruuz]: punane vein
le vin blanc [vÄ~ 'blA~]: valge vein
le vin rosé [vÄ~ ro'zee] : roosa vein
le cidre ['siidrə] : siider
la région [rež'jO~] : regioon, ala
le terroir [ter'waar] : geograafiline veinirühm
une appellation d'origine contrôlée (AOC) [apela'sjO~ dori'žin kO~trO'lee] : kontrollitud päritolumaa nimetus

la bavaroise [bavaru'aaz] : jook teest, piimast ja liköörist
""")

Kokadele.add_lesson(u"Menüü", intro=u"""""")
Kokadele.parse_words(Nom,u"""
le plat [pla] : roog 
le plat du jour [pla dü žuur] : päevapraad
le *hors d'œuvre [OOr 'dÖÖvrə] : eelroog
le dessert [des'säär] : magustoit
""")

Kokadele.add_lesson(u"Supid", u"""
""")
Kokadele.parse_words(Nom,u"""
la soupe [sup] : supp 
le potage [po'taažə] : juurviljasupp
le potage purée [po'taažə pü'ree] : püreesupp
le velouté [vəlu'tee] : koorene püreesupp
le velouté Dubarry [vəlu'tee düba'ri] : koorene püreesupp lillkapsaga
le bouillon [bui'jO~] :  puljong 
le consommé [kO~som'mee] : selge puljong 
le consommé de volaille [kO~som'mee də vo'lajə] : linnulihast puljong 
le consommé de gibier [kO~som'mee də žib'jee] : ulukilihast puljong 
le consommé de poisson [kO~som'mee də pwa'sO~] : kala puljong 
le consommé double [kO~som'mee 'duublə] : kahekordne puljong 
#rammuleem?
""")



Kokadele.add_lesson(u"Liha", u"""
""")
Kokadele.parse_words(Nom,u"""
la viande [vjA~də] : liha
la volaille [vo'lajə] : linnuliha
le gibier [žibiee] : jahiloomad
le poisson [pwa'sO~] : kala

la boucherie [bušə'rii] : lihakauplus, lihakarn
le lard [laar] : pekk
le jambon [žA~'bO~] : sink
la saucisse [soo'sis] : vorst

un os [os] : kont
la côte [koot] : ribi
le dos [do] : selg
la langue [lA~gə] : keel
le foie [fwa] : maks
les tripes [trip] : soolestik
le cœur [kÖÖr] : süda
le rognon [ron'jO~] : neer
la cervelle [ser'vell] : aju
les abats [a'ba] : subproduktid (maks,süda, neerud, keel, jalad)
""")



Kokadele.add_lesson(u"Liharoad", u"""""")
Kokadele.parse_words(Nom,u"""
l'escalope (f) [eska'lOp] : eskalopp, šnitsel
le ragoût [ra'gu] : raguu
la roulade [ru'laadə] : rulaad
la paupiette [pop'jät] : liharull
l'aspic (m) [as'pik] : sült
le filet [fi'lä] : filee | võrk
le bifteck [bif'täk] : biifsteek
la brochette [bro'šät] : lihavarras, šašlõk
les attereaux [attə'roo] : fritüüris praetud varras, paneeritud šašlõk
la côtelette [kot'lät] : naturaalne kotlet
la côtelette de porc [kot'lät də pOOr] : sealiha kotlet
la noisette de porc [nwa'zät də pOOr] : filee sealiha
le goulasch [gu'laš] : guljašš
le *hachis [ha'ši] : hakkliha
la boulette [bu'lett] : lihapall
le tournedos [turnə'do] : veise sisefilee portsjon toode
la entrecôte [A~trə'koot] : antrekoot
le Chateaubriand [šatobri'A~] : praetud liharoog
le carré d'agneau [ka'ree dan'joo] : praetud tallerind
la poitrine d'agneau farcie [pwa'trin dan'joo far'sii] : täidetud tallerind
le cœur de filet [kÖÖr də fi'lä] : veise sisefilee
le filet mignon [fi'lä min'jO~] : veise sisefilee portsjon toode
le filet médaillon [fi'lä meda'jO~] : medaljon (veise sisefilee portsjon toode)
le médaillon de veau [meda'jO~ də'voo] : vasika medaljon
le bœuf bourguignon [bÖff burgin'jO~] : härjapraad burgundia veiniga
le bœuf à la tartare [bÖff a la tar'taar] : väiketükiline toode sisefileest
le bœuf à la Strogonov [bÖff a la strogo'nov] : böfstrogonov
le sauté de bœuf à la suédoise [so'tee də bÖff a la süee'dwaazə] : klopsid
le sauté de veau [so'tee də voo] : pajaroog vasikalihast
la selle de mouton [säl də mu'tO~] : lamba (talle) sadul
""")


Kokadele.add_lesson(u"Road", intro=u"""""")
Kokadele.parse_words(Nom,u"""
la purée [pü'ree] : püree
un œuf [Öf] : muna
les œufs brouillés [öö brui'jee] : omlett
les œufs pochés [öö po'šee] : ilma kooreta keedetud muna
le gratin [gra'tÄ~] : gratään (ahjus üleküpsetatud roog)
le gratin dauphinois [gra'tÄ~ dofinw'a] : (tuntud retsept)
le gratin savoyard [gra'tÄ~ savwa'jaar] : (juustuga gratin dauphinois)
le soufflé [suff'lee] : suflee
la quiche lorraine [kiš lo'rään] : quiche
la pâte brisée [paat bri'zee] : (Mürbeteig, shortcrust pastry)
la tourte [turt] : (ingl. *pie*)

la fondue [fO~'düü] : fondüü
le fumet [fü'mä] : hea lõhn (nt. veini, liha kohta)
le pâté [pa'tee] : pasteet
le pâté en croûte [pa'tee A~'krut] : küpsetatud taignas pasteet
le pâté en terrine [pa'tee A~ter'rin] : küpsetatud pasteet kausis
la galantine [galA~'tin] : galantiin
le cassoulet [kasu'lä] : Languedoc'ist pärit ühepajatoit ubadest ja lihast, mida küpsetatakse mitu tundi madala temperatuuriga ahjus.
le pot-au-feu [poto'föö] : ühepajatoit
""")



Kokadele.add_lesson(u"Juust", u"""""")
Kokadele.parse_words(None,u"""
le fromage [fro'maaž] : juust
la caillebotte [kajə'bott] : (kodujuust)
la raclette [rak'lett] : kuumaga sulatud juust
le Camembert [kamA~'bäär] : (valgehallitusjuust)
le Emmental [emən'taal] : -
le Rocquefort [rOk'fOOr] : (sinihallitusjuust)
le Gruyère [grüi'jäär] : -
le Edam [e'dam] : -
le Brie [brii] : -
le Fontal [fO~'tal] : -
le Parmesan [parmə'zA~] : -
le Gouda [gu'da] : (holandi juust)
le Aura [o'ra] : -
""")


Kokadele.add_lesson(u"Magustoidud", u"""""")
Kokadele.parse_words(Nom,u"""
le dessert [des'säär] : magustoit
la crème [krääm] : koor
la crème fraiche [krääm 'fräš] : rõõsk koor
la crème brûlée [krääm brü'lee] : põletud koor
la crème bavaroise [krääm bavaru'aaz] : muna-piima-seguga kreem želatiiniga 
la sauce melba [soos mel'ba] : melba kaste
la sauce vanille [soos va'niijə] : vanillikaste
la sauce caramel [soos kara'mäl] : karamellkaste
la crêpe [kräp] : pannkook
la glace [glass] : jäätis
le sorbet [sor'bä] : jäätis (ilma kooreta)
le parfait [par'fä] : parfee
le gâteau [ga'too] : kook
la gaufre ['goofrə] : vahvel
la tarte [tart] : tort
la compote [kO~'pOt] : kompott
la gelée [žə'lee] : tarretis
la confiture [kO~fi'tüür] : moos
la mousse [mus] : vaht
la tarte aux prunes [tarto'prün] : ploomikook
la salade de fruits [sa'laad də frü'i] : puuviljasalat
la salade de baies [sa'laad də bä] : marjasalat
un petit-beurre [pəti'bÖÖr]: (kuiv küpsis)
""")



Kokadele.add_lesson(u"Puuviljad", u"""""")
Kokadele.parse_words(Nom,u"""
le fruit [frü'i] : puuvili
un ananas [ana'na] : ananass 
la banane [ba'nan] : banaan
le citron [si'trO~] : sidrun
une orange [o'rA~ž] : apelsin
la pomme [pom] : õun
la poire [pu'aar] : pirn
la prune [prünn] : ploom
la cerise [sə'riiz] : kirss
la noix [nwa] : pähkel
la noisette [nwa'zett] : sarapuupähkel
""")

Kokadele.add_lesson(u"Marjad", u"""""")
Kokadele.parse_words(Nom,u"""
la baie [bä] : mari
la fraise [frääz] : maasikas
la myrtille [mir'tiijə] : mustikas
la mûre [müür] : põldmari
la groseille [gro'zeijə] : sõstar (punane v. valge) | tikker
le cassis [ka'sis] : mustsõstar 
""")


Kokadele.add_lesson(u"Juurviljad", u"""""")
Kokadele.parse_words(Nom,u"""
la légume [le'güm] : juurvili
la pomme de terre [pom də 'täär] : kartul
la tomate [to'mat] : tomat
la carotte [ka'rOt] : porgand
# la betterave []
# le panais
# le radis
# le salsifis
# le cerfeuil
la asperge [as'pärž] : spargel
le épinard [epi'naar] : spinat
le concombre [kO~kO~brə]: kurk
un *haricot [ari'ko] : uba
la salade [sa'laadə] : salat
la endive [A~'diiv] : endiiv
# le chicon [ši'kO~] : 
le chou [šu] : kapsas
le chou-fleur [šu 'flÖÖr] : lillkapsas
""")

Kokadele.add_lesson(u"Teraviljad", u"""""")
Kokadele.parse_words(Nom,u"""
le blé [blee] : teravili
l'avoine (f) [avu'ann] : kaer
le froment [fro'mA~] : nisu
le sarrasin [sara'zÄ~] : tatar
le blé noir  [blee'nwaar] : tatar
le riz [ri] : riis
le seigle ['sääglə] : rukis
l'orge (m) ['Oržə] : oder
""")

Kokadele.add_lesson(u"Teraviljatooded", u"""""")
Kokadele.parse_words(Nom,u"""
le riz pilaf [ri pi'laf] : pilaff
les pâtes ['paat] : pastaroad
la farine [far'in] : jahu
la bouillie [bui'jii] : puder
le gruau [grü'oo] : puder
le pain [pÄ~] : sai | leib
la tartine [tar'tin] : võileib
la baguette [ba'gät] : prantsuse pikk sai
le croustillon [krusti'jO~] : õlis praetud kohupiimapall
le crouton [kru'tO~] : krutoon
""")


Kokadele.add_lesson(u"Koostisosad", u"""""")
Kokadele.parse_words(Nom,u"""
le ingrédient [Ä~gre'djA~] : koostisosa 
le lait [lä] : piim
le beurre [bÖÖr]: või
la crème [kr’ääm] : kreem | koor
le sucre ['sükrə] : suhkur
le sel [säl] : sool
le poivre ['pwaavrə] : pipar
""")


Kokadele.add_lesson(u"Ürdid", u"""""")
Kokadele.parse_words(Nom,u"""
un assaisonnement [asäzon'mA~] : maitsestamine
le condiment [kO~di'mA~] : maitseaine
une épice [e'pis] : vürts
les fines herbes [fin'zärbə] : fines herbes ("peened ürdid")
une herbe [ärbə] : ürt
le persil [pär'sil] : petersell
le céléri [sele'ri] : seller
la gousse d'ail [guss 'daij] : küüslaugu küün
l'ail (m) [aj] : küüslauk
un oignon [on'jO~] : sibul
la ciboulette [sibu'lät] : murulauk
la câpre ['kaaprə] : kappar
le gingembre [žÄ~žA~brə] : ingver
""")


Kokadele.add_lesson(u"Köögiriistad", u"""""")
Kokadele.parse_words(Nom,u"""
la cuisine [kwi'zin] : köök
la cuisinière [kwizin'jäär] : pliit
le four [fuur] : ahi
le four à micro-ondes [fuur a mikro 'O~də] : mikrolaine ahi
le fouet [fu'ä] : vispel
le moulin [mu'lÄ~] : veski
la alumette [alu'mät] : tuletikk
la coquille [ko'kiijə] : merekarp
la cocotte [ko'kot] : malmkastrul, kokott
la poêle [pwal] : pann
la râpe [rap] : riiv
la casserole [kas'roll] : kastrul
la marmite [mar'mit] : katel
la braisière [bräz'jäär] : pott smoorimiseks 
le caquelon [kak'lO~] : fondüüpott
le bain-marie [bÄ~ma'rii] : veevann
""")


Kokadele.add_lesson(u"Mida kokk teeb", intro=u"""
""")
Kokadele.parse_words(Verbe,u"""
préparer [prepa'ree] : ette valmistama
# composer [kO~po'zee] : koostama
# baisser [bäs'see] : alla laskma, madaldama
# porter [por'tee] : kandma
laver [la'vee] : pesema
concasser [kO~kas'see] : peenestama (tükkideks)
farcir [far'siir] : farssima (täidisega täitma)
*hacher [a'šee] : hakkima
éplucher [eplü'šee]: koorima
émincer [émÄ~'see] : lõikama viiludeks
tourner [tur'nee] : keerama, pöörama
# utiliser [ütili'zee] : kasutama
préchauffer [préšoo'fee] : ette kütma
""")


Kokadele.add_lesson(u"Pliidil", u"""""")
Kokadele.parse_words(Nom,u"""
la cuisson [küis'sO~] : keetmine
le blanchiment [blA~ši'mA~] : blanšeerimine
le rôtissage [rotis'saaž] : praadimine (panni peal)
le rissolement [risol'mA~] : praadimine 
la friture [fri'tüür] : friipraadimine (õlis või rasvas)
le grillage [gri'jaaž] : röstimine
le braisage [bre'zaaž] : smoorimine
""")

#~ le bain marie [bÄ~ ma'rii] : 

Kokadele.parse_words(Verbe,u"""
cuire [kwiir] : keetma
blanchir [blA~'šiir] : blanšeerima
rôtir [ro'tiir] : praadima (panni peal)
rissoler [risso'lee]: (rasvas) pruunistama
frire [friir] :  praadima (õlis)
griller [gri'jee] : röstima
braiser [brä'zee] : smoorima
""")







u"""
On met une majuscule 
uniquement quand l’adjectif est employé comme 
nom pour désigner une personne. 
Ex. : Les Français parlent en français à leurs amis français
"""
General.add_lesson(u"Riigid",columns=[GEON("Riik"), GEOM, GEOF, ET])
General.parse_words(None,u"""
la France [frA~s] | français [fra~'sä] | française [fra~'sääz] : Prantsusmaa
l'Estonie (f) [ästo'nii] | estonien [esto'njÄ~] | estonienne [esto'njän] : Eesti
l'Allemagne (f) [al'manjə] | allemand [al'mA~]| allemande [al'mA~də] : Saksamaa
l'Angleterre (f) [A~glə'täär] | anglais [A~'glä]| anglaise [A~'glääz] : Inglismaa
la Belgique [bel'žik] | belge [belžə]| belge [belžə] : Belgia
la *Hollande [o'lA~də] | hollandais [olA~'dä] | hollandaise [olA~'dääz] : Holland
l'Espagne (f) [es'panjə] | espagnol [espan'jol] | espagnole [espan'jol] : Hispaania
l'Italie (f) [ita'lii] | italien [ital'jÄ~]| italienne [ital'jen] : Itaalia
""")

if FULL_CONTENT:
    General.add_lesson(u"Linnad prantsusmaal",columns=[GEON("Linn"), GEOM, GEOF])
    General.parse_words(NomGeographique,u"""
    Avignon [avin'jO~] | avignonnais [avinjo'nä] | avignonnaise [avinjo'nääz] : -
    Bordeaux [bor'doo] | bordelais [bordə'lä] | bordelaise [bordə'lääz] : -
    Bourgogne [burgOnjə] | bourguignon [burgin'jO~] | bourguignonne [burgin'jOnn] : -
    Dijon [di'žO~] | dijonnais [dižon'nä] | dijonnaise [dižon'nääz] : -
    Lyon [li'O~] | lyonnais [lio'nä] | lyonnaise [lio'nääzə] : -
    Marseilles [mar'säijə] | marseillais [marsäi'jä]| marseillaise [marsäi'jääz] : - 
    Paris [pa'ri] | parisien [pariz'jÄ~]| parisienne [pariz'jän] : Pariis
    Reims [rÄ~s] | rémois [rem'wa]| rémoise [rem'waaz] : Reims
    Verdun [vär'dÖ~] | verdunois [värdü'nwa]| verdunoise [värdü'nwaaz] : -
    Versailles [ver'saj] | versaillais [värsa'jä] | versaillaise [värsa'jääz] : -
    """)
#~ Vocabulary.parse_words(NomGeographique,u"""
#~ la France [frA~s] : Prantsusmaa
#~ la Belgique [bel'žik] : Belgia
#~ une Allemagne [al'manjə] : Saksamaa
#~ une Angleterre [A~glə'täär] : Inglismaa
#~ une Estonie [ästo'nii]: Eesti
#~ une Hollande [o'lA~də]: holland
#~ une Espagne [es'panjə]: hispaania
#~ une Italie [ita'lii]: hispaania
#~ """)
#~ Vocabulary.parse_words(Adjectif,u"""
#~ français [fra~'sä] | français [fra~'sääz] : prantsuse
#~ estonien [esto'njÄ~] | estonien [esto'njän] : eesti
#~ espagnol [espan'jol] | espagnole [espan'jol] : hispaania
#~ hollandais [olA~'dä] | hollandaise [olA~'dääz]: holandi
#~ """)

Kokadele.add_lesson(u"Omadussõnad (kulinaaria)", intro=u"""
Selliseid omadussõnu leidub erinevates kulinaaria väljundites.
""",columns=[M, F, ET])
Kokadele.parse_words(Adjectif,u"""
beurré [bÖÖ'ree] | beurrée [bÖÖ'ree]: võiga
bouilli [bui'ji] | bouillie [bui'jii] : keedetud
braisé [brä'zee] | braisée [brä'zee] : smooritud
coupé [ku'pee] | coupée [ku'pee] : lõigatud
épicé [epi'see] | épicée [epi'see] : vürtsitatud, vürtsikas
glacé [glas'see] | glacée [glas'see] : jäätunud
haché [a'šee] | hachée [a'šee] : hakkitud
manié [man'jee] | maniée [man'jee] : käsitletud
poché [po'še] | pochée [po'šee] : uputatud keeva vette
rissolé [riso'lee] | rissolée [riso'lee] :  (rasvas) pruunistatud
sauté [soo'tee] | sautée [soo'tee] : rasvas praetud
velouté [velu'tee] | veloutée [velu'tee] : sametine, sametitaoline
croustillant [krusti'jA~] | croustillante [krusti'jA~t] : krõbe
gourmand [gur'mA~] | gourmande [gur'mA~d] : maiasmokk
paysan [pei'zA~] | paysanne [pei'zann] : talu-, talupoja-
royal [rwa'jal] | royale [rwa'jal] : kuninglik
suprême (mf) [sü'prääm] : ülem, kõrgem, ülim
""")




Kokadele.add_lesson(u"Kastmete valmistamine", intro=u"""""")
Kokadele.parse_words(Nom,u"""
la sauce [soos] : kaste

la moutarde [mu'tardə] : sinep
le vinaîgre [vin'äägrə] : äädikas
la mayonnaise [majo'nääz] : majonees
la vinaîgrette [vine'grät] : vinegrett

le beurre manié [bÖÖr man'jee]: jahuvõi
le roux [ru] : rasvas kuumutatud jahu
le roux blanc [ru blA~] : valge segu
le roux blond [ru blO~] : kollane segu
le roux brun [ru brÖ~] : pruun segu
la sauce espagnole [espan'jOl] : pruun põhikaste
la sauce demi-glace [dəmi'glas] : redutseeritud pruun põhikaste
le jus de rôti [žu də ro'ti] : redutseeritud puljong, "praeliha mahl"
le jus lié [žu li'ee] : maisi või nooljuurejahuga pruun kaste

le mirepoix [mirə'pwa] : praetud kuubikud (sibul, porgand, seller)

la coupe en brunoise [kup A~ brün'waaz] : juur- või puuvilja kuubikud (2mm)
la coupe julienne [kup jül'jän] : juurvilja lõikamine ribadeks (2mm)
la coupe jardinière [kup žardin'jäär] : juurvilja lõikamine ribadeks
la coupe à la paysanne [kup ala päi'zan] : juurvilja lõikamine ketasteks
""")

Kokadele.add_lesson(u"Kastmed", intro=u"""""")
Kokadele.parse_words(Nom,u"""
la sauce paysanne [pei'zan] : talupoja kaste
la sauce chasseur [ša'sÖÖr] : jahimehe kaste
la sauce jardinière [žardin'jäär] : aedniku kaste

la sauce piquante [pi'kA~tə] : pikantne kaste
la sauce poivrade [pwav'raadə] : piprakaste

la sauce Grand Veneur [grA~ və'nÖÖr] : jäägri kaste
la sauce Bigarrade [biga'raadə] : apelsinikaste
la sauce smitane [smi'tanə] : hapukoorekaste
la sauce Lyonnaise [lio'nääzə] : pruun sibulakaste
la sauce Bourguignonne [burgin'jOn] : Burgundia kaste
la sauce Robert [ro'bäär] : Roberti kaste
la sauce Madère [ma'däär] : Madeira kaste
la sauce Mornay [mOr'nä] : juustukaste
la sauce Porto [pOr'to] : portveini kaste
la sauce Sabayon [saba'jO~] : Sabayon-kaste
la sauce italienne [ital'jän] : itaalia kaste
la sauce veloutée [vəlu'tee] : hele põhikaste
la sauce blanche [blA~šə] : tuletatud hele kaste
la sauce bordelaise [bOrdə'lääz] : punase veini kaste
la sauce béarnaise [bear'nääz] : bernoo kaste

la sauce béchamel [beša'mäl] : valge põhikaste
la sauce aurore [o'rOOr] : aurorakaste
la sauce Choron [šo'rO~] : choronkaste
la sauce Foyot [fwa'jo] : foyotkaste

la macédoine [mase'dwan] : juurviljasalat
""")

Kokadele.add_lesson(u"Veinialad Prantsusmaal", u"""
""",columns=[FR,PRON])
Kokadele.parse_words(NomGeographique,u"""
Alsace [al'zas] : Elsass
Beaujolais [boožo'lä] : -
Bordeaux [bOr'doo] : -
Bourgogne [bur'gonjə] : -
Champagne [šA~'panjə] : -
Charente [ša'rA~tə] : -
Poitou [pwa'tu] : -
Corse ['korsə] : Korsika
Jura [žü'ra] : -
Savoie [savu'a] : -
Languedoc [lA~gə'dok] : -
Roussillon [russi'jO~] : -
Provence [pro'vA~sə] : - 
Sud-Ouest [süd'uest] : - 
Gascogne [gas'konjə] : -
Val de Loire [val də 'lwaarə] : Loire'i org
Vallée du Rhône [val'lee dü roonə] : Rhône'i org
""")

Kokadele.add_lesson(u"Prantsuse veinid", u"""
Prantsuse veinid on üle 400, siin ainult mõned.
""",columns=[FR,PRON])
Kokadele.parse_words(Nom,u"""
le Chasselas [šas'la] : -
le Grand Cru [grA~'krü] : - 
le Pinot Noir [pi'no nwaar] : - 
la Côte de Brouilly [koot də bru'ji] : -
le Saint-Amour [sÄ~ta'muur] : - 
le Bordeaux clairet [bOr'doo klä'rä] : - 
le Médoc [me'dok] : - 
le Saint-Émilion [sÄ~temi'jO~] : - 
la Côte de Beaune [koot də boon] : - 
les Côtes du Ventoux [kootə du vA~'tu] : -
le Minervois [minerv'wa] : - 
les Côtes d'Auvergne [kootə do'värnjə] : -
""")




if FULL_CONTENT:

    General.add_lesson(u"Omadussõnad (üld)", intro=u"""
    Omadussõnad, mis lõpevad "e"-ga, ei muutu soo järgi.
    """,columns=[M, F, ET])
    General.parse_words(Adjectif,u"""
    chaud [šoo] | chaude [šoodə] : kuum
    froid [fru'a] | froide [fru'aadə] : külm
    gros [gro] | grosse [grossə] : paks
    mince (mf) [mÄ~s] : õhuke
    bon [bO~] | bonne [bonnə] : hea
    beau [boo] | belle [bellə] : ilus
    joli [žo'li] | jolie [žo'lii] : ilus
    demi [də'mi] | demie [də'mii]: pool
    entier [A~'tjee] | entière [A~'tjäär] : terve, täis
    double (mf) ['duublə] : topelt
    facile (mf) [fa'sil] : lihtne
    possible (mf) [po'siblə] : võimalik
    """)



    General.add_lesson(u"Loeme kümneni", intro=u"""""")
    General.parse_words(Numerique,u"""
    un [Ö~] : üks
    deux [döö] : kaks
    trois [trwa] : kolm
    quatre [katrə] : neli
    cinq [sÄ~k] : viis
    six [sis] : kuus
    sept [sät] : seitse
    huit [üit] : kaheksa
    neuf [nÖf] : üheksa
    dix [dis] : kümme
    """)


    General.add_lesson(u"Värvid", columns=[M, F, ET])
    General.parse_words(Adjectif,u"""
    brun [brÖ~] | brune [brünn] : pruun
    vert [väär] | verte [värtə] : roheline
    bleu [blöö] | bleue [blöö] : sinine
    rouge (mf) [ruuž] : punane
    jaune (mf) [žoon] : kollane
    blond [blO~] | blonde [blO~də] : blond 
    beige (mf) [bääž]  : beež
    orange (mf) [o'ra~ž]  : oranž
    blanc [blA~] | blanche [blA~š] : valge
    noir [nwaar] | noire [nwaar] : must
    """)


if FULL_CONTENT:
    General.add_lesson(u"Kuulsad inimesed")
    General.parse_words(NomPropre,u"""
    Jacques Chirac [žaak ši'rak] : # endine president
    Georges Brassens [žorž bra'sÄ~s] : # laulja
    Brigitte Bardot [bri'žit bar'do] : # laulja
    Louis de Funès [lu'i də fü'nääz] : # näitleja
    """)

    General.add_lesson(u"Kuud")
    General.parse_words(NomPropre,u"""
    janvier [žA~'vjee] : jaanuar
    février [fevri'ee] : veebruar
    mars [mars] : märts
    avril [a'vril] : aprill
    mai [mä] : mai
    juin [žwÄ~] : juuni
    juillet [žwi'jä] : juuli
    août [ut] : august
    septembre [sep'tA~brə] : september
    octobre [ok'tOObrə] : oktoober
    novembre [no'vA~brə] : november
    décembre [de'sA~brə] : detsember
    """)

    General.add_lesson(u"Majad ja nende osad")
    General.parse_words(Nom,u"""
    la maison [mä'zO~] : maja
    la cave [kaav] : kelder
    la cuisine [kwi'zin] : köök
    la salle de bain : vannituba
    la chambre à coucher : magamistuba
    le salon [sa’lO~] : elutuba
    un escalier [eskal'jee] : trepp
    la fenêtre [fə'näätrə] : aken
    le parterre [par'täär] : esimene korrus
    le premier étage [prəm'jeer_etaaž] : teine korrus
    le jardin [žar'dÄ~] : aed
    """)

if FULL_CONTENT:


    Fun.add_lesson(u"Virelangues", intro=u"""
    #. Poisson sans boisson est poison. 
    #. Un chasseur sachant chasser doit savoir chasser sans son chien. 
    #. Ecartons ton carton, car ton carton me gêne. 
    #. Ton thé t'a-t-il ôté ta toux? 
    #. Chacun cherche son chat.
    #. Tante, en ton temps teintais-tu tes tempes?
    #. Les poules couvent souvent au couvent.

    """)
    Fun.parse_words(Nom,u"""
    le poisson [pwa'sO~] : kala
    le poison [pwa'zO~] : mürk
    la boisson [bwa'sO~] : jook
    le chasseur [ša'sÖÖr] : jahimees
    le chien [šiÄ~] : koer
    la toux [tu] : köha
    """)
    Fun.parse_words(Verbe,u"""
    savoir [sa'vuaar] : teadma | oskama
    chercher [šär'šee] : otsima
    écarter [ekar'tee] : eest ära liigutama
    ôter [oo'tee] : ära võtma
    """)
    Fun.parse_words(Autre,u"""
    sans [sA~] : ilma
    chacun [ža'kÖ~] : igaüks
    """)


if FULL_CONTENT:

    General.add_lesson(u"Lisa", intro=u"""
    """)
    General.parse_words(Autre,u"""
    environ [A~vi'rO~] : umbes
    facilement [fasil'mA~] : lihtsalt
    rapidement [rapidə'mA~]: kiiresti
    le autre [ootrə] : teine
    le même [määm] : sama
    """)

    General.parse_words(Verbe,u"""
    filer [fi'lee] : ketrama
    baiser [bä'zee] : musitama
    sauter [soo'tee] : hüppama
    """)

    General.parse_words(Nom,u"""
    le midi [mi'di] : lõun | keskpäev
    le soir [swaar] : õhtu
    le matin [ma'tÄ~] : hommik
    la tranche [trA~š] : lõik | viilukas
    la coupe [kupp] : lõikamine | pokaal
    une ébullition [ebüjis'jO~] : keemine
    le feu [föö] : tuli
    le baiser [bä'zee] : suudlus
    un appétit [appe'ti] : isu
    """)

    unused = u"""
    une aurore [or'Or] : koit
    le fil [fil] : niit | lõng | nöör
    une heure [ÖÖr] : tund
    le dauphinois [dofinw'a] : lõunaprantsuse dialekt
    """

Pronounciation.add_lesson(u"Spikker", intro=u"""

Hääldamisreeglid:

[ruleslist ai
ail
ain
an
au
c
cedille
ch
eil
en
ent
eu
g
gn 
gu
h
ien
il
ill 
in
j
oi
oin
on
ou
u
un
ueil
ui 
y]

""")



if __name__ == '__main__':
    if fmt == "rst":
        book.add_index(u"Sõnaraamat")
        book.write_rst_files(sys.argv[2])
    elif fmt == "odt":
        book.add_dictionary(u"Sõnade nimekiri")
        fn = sys.argv[2]
        book.write_odt_file(fn)
        os.startfile(fn)



## Copyright 2012 Luc Saffre
## This file is part of the Lino project.
## Lino is free software; you can redistribute it and/or modify 
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## Lino is distributed in the hope that it will be useful, 
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with Lino; if not, see <http://www.gnu.org/licenses/>.

import logging
logger = logging.getLogger(__name__)

#~ from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from lino.utils import choicelists
from lino.utils import curry
from lino.core.modeltools import obj2str


class UserLevels(choicelists.ChoiceList):
    """
    The level of a user is one way of differenciating users when 
    giving access permissions. User levels are a graduation: 
    a "Manager" is higher than a simple "User" and thus 
    can do everything for which a simple "User" level has permission.
    
    About the difference between "Administrator" and "Manager":
    
    - "Management is closer to the employees. 
      Admin is over the management and more over the money 
      of the organization and lilscencing of an organization. 
      Mananagement manages employees. 
      Admin manages the outside contacts and the 
      facitlity as a whole." (`answerbag.com <http://www.answerbag.com/q_view/295182>`__)
    
    - See also a more detailed overview at
      http://www.differencebetween.com/difference-between-manager-and-vs-administrator/
    
    """
    label = _("User Level")
    
    @classmethod
    def field(cls,module_name=None,**kw):
        """
        Shortcut to create a :class:`lino.core.fields.ChoiceListField` in a Model.
        """
        kw.setdefault('blank',True)
        if module_name is not None:
            kw.update(verbose_name=string_concat(cls.label,' (',module_name,')'))
        return super(UserLevels,cls).field(**kw)
        
add = UserLevels.add_item
add('10', _("Guest"),'guest')
add('20', _("Restricted"),'restricted')
add('30', _("User"), "user")
add('40', _("Manager"), "manager")
add('50', _("Administrator"), "admin")
add('90', _("Expert"), "expert")

class UserGroups(choicelists.ChoiceList):
    """
    This list is cleared and re-filled in lino.apps.pcsw.models. 
    applications may want to clear() this list and add longer 
    names than the only default "system".
    Since this redefinitions happens at a moment where database 
    fields have already been instantiated, 
    it is too late to change their max_length.
    """
    label = _("User Group")
    show_values = True
    max_length = 20 
    """
    """
    
        
#~ add = UserGroups.add_item
#~ add('system', _("System"))

SHORT_LEVELS = dict(A='admin',U='user',_='',M='manager',G='guest')

class UserProfile(choicelists.Choice):
  
    def __init__(self,cls,value,text,name=None,memberships=None,readonly=False,**kw):
      
        super(UserProfile,self).__init__(cls,value,text,name)
        
        #~ keys = ['level'] + [g+'_level' for g in choicelist.groups_list]
        #~ keys = ['level'] + [g+'_level' for g in choicelist.membership_keys]
        self.readonly = readonly

        if memberships is None:
            for k in cls.membership_keys:
                #~ kw[k] = UserLevels.blank_item
                kw.setdefault(k,UserLevels.blank_item)
        else:
        #~ if memberships is not None:
            if len(memberships.split()) != len(cls.membership_keys):
                raise Exception(
                    "Invalid memberships specification %r : must contain %d letters" 
                    % (memberships,len(cls.membership_keys)))
            for i,k in enumerate(memberships.split()):
                kw[cls.membership_keys[i]] = UserLevels.get_by_name(SHORT_LEVELS[k])
                
        #~ print 20120705, value, kw
        
        assert kw.has_key('level')
            
        for k,v in kw.items():
            setattr(self,k,v)
            
        
        #~ for grp in enumerate(UserGroups.items()):
            #~ attname = grp.value + '_level'
            #~ setattr(self,attname,kw.pop(attname,''))
        #~ if kw:
            #~ raise Exception("UserProfile got unexpected arguments %s" % kw)
            
        #~ dd.UserProfiles.add_item(value,label,None,**kw)
      
    #~ def __init__(self,level='',*args,**kw):
    #~ def __init__(self,level='',*args):
    #~ def __init__(self,**kw):
        #~ if level:
            #~ self.level = getattr(UserLevels,level)
        #~ else:
            #~ self.level = UserLevels.blank_item
        #~ self.level = UserLevels.get_by_name(level)
        #~ groups = UserGroups.items()
        #~ if len(args) > len(groups):
            #~ raise Exception("More arguments than user groups.")
        #~ for i,levelname in enumerate(args):
            #~ attname = groups[i].value + '_level'
            #~ v = UserLevels.get_by_name(levelname)
            #~ setattr(self,attname,v)
        #~ kw.setdefault('readonly',False)
        #~ for k,v in kw.items():
            #~ setattr(self,k,v)
            
    def __repr__(self):
        s = self.__class__.__name__ + ":" + self.value + "("
        s += "level=%s" % self.level.name
        for g in UserGroups.items():
            if g.value: # no level for UserGroups.blank_item
                v = getattr(self,g.value+'_level',None)
                if v is not None:
                    s += ",%s=%s" % (g.value,v.name)
        s += ")"
        return s
        

        
class UserProfiles(choicelists.ChoiceList):
    """
    
    """
    #~ item_class = UserProfile
    label = _("User Profile")
    show_values = True
    max_length = 20
    membership_keys = ('level',)
    
    #~ @classmethod
    #~ def clear(cls):
        #~ cls.groups_list = [g.value for g in UserGroups.items()]
        #~ super(UserProfiles,cls).clear()
          

    #~ @classmethod
    #~ def clear(cls,groups='*'):
    @classmethod
    def reset(cls,groups):
        #~ cls.groups_list = [g.value for g in UserGroups.items()]
        s = []
        expected_names = set(['*']+[g.value for g in UserGroups.items() if g.value])
        for g in groups.split():
            if not g in expected_names:
                raise Exception("Unexpected name %r (UserGroups are: %s)" % (
                    g,[g.value for g in UserGroups.items() if g.value]))
            else:
                expected_names.remove(g)
                if g == '*':
                    s.append('level')
                else:
                    if not UserGroups.get_by_value(g):
                        raise Exception("Unknown group %r" % g)
                    s.append(g+'_level')
        if len(expected_names) > 0:
            raise Exception("Missing name(s) %s in %r" % (expected_names,groups))
        cls.membership_keys = tuple(s)
        cls.clear()

    @classmethod
    def add_item(cls,value,text,memberships=None,name=None,**kw):
        return cls.add_item_instance(UserProfile(cls,value,text,name,memberships,**kw))
          
    
add = UserProfiles.add_item
add('100', _("User"), name='user', level='user')
add('900', _("Administrator"), name='admin', level='admin')


class Permittable(object):  
    """
    Base class for Components that have permissions control.
    """
    required = {}
    """
    Conditions required to read (view) this component.
    """
    workflow_state_field = None # internally needed for make_permission_handler
    workflow_owner_field = None # internally needed for make_permission_handler
    #~ readonly = True
    
    
    #~ def allow_read(self,user,obj,state):
        #~ return True
    
    def __init__(self,actor):
        #~ super(PermissionComponent,self).__init__(self,*args,**kw)
        self.required = {}
        self.allow_read = curry(make_permission_handler(
            self,self,True,
            actor.debug_permissions,
            **self.required),self)

        
    def get_view_permission(self,user):
        #~ logger.info("20120622 %s.get_view_permission",self)
        return self.allow_read(user,None,None)
        
        



#~ BLANK_STATE = ''


def make_permission_handler(
    elem,actor,
    readonly,debug_permissions,
    user_level=None,user_groups=None,states=None,allow=None,owner=None,auth=False):
    """
    Return a function that will test whether permission is given or not.
    
    `elem` is either an Action or a Permittable.
    `readonly`
    
    The generated function will always expect three arguments user, obj and state.
    The latter two may be None depending on the context
    (for example a read_required is expected to not test on obj or 
    state because these values are not known when generating the 
    :xfile:`lino*.js` files.).
    
    `user_level`
        A string (e.g. ``'manager'``, ``'user'``,...) 
        The minimum :class:`user level <lino.utils.choicelists.UserLevels>` 
        required to get the permission.
        The default value `None` means that no special user level is required.
        
    `user_groups`
        List of strings naming the user groups for which membership is required 
        to get permission to view this Actor.
        The default value `None` means that no special group membership is required.
        Alternatively, if this is a string, it will be converted to a list of strings.
        
    `states`
        List of strings naming the user groups for which membership is required 
    
    `allow`
        An additional custom permission handler
        
    `auth`
        If True, permission is given for any authenticated user 
        (and not for :class:`lino.utils.auth.AnonymousUser`).
        
    `owner`
        If True, permission is given only to the author of the object. 
        If False, permission is given only to users who are not the author of the object. 
        To be used on models that have a field `user` which 
        contains the "owner" (usually a subclass of UserAuthored,
        but e.g. :class:`lino.modlib.cal.models.Guest` 
        defines a property `user` because it has no own `user` field).
    
    
    """
    try:
        if allow is None:
            def allow(action,user,obj,state):
                return True
        if auth:
            allow_before_auth = allow
            def allow(action,user,obj,state):
                if not user.authenticated:
                    return False
                return allow_before_auth(action,user,obj,state)
                
        if user_level is not None:
            user_level = getattr(UserLevels,user_level)
            allow_user_level = allow
            def allow(action,user,obj,state):
                #~ if user.profile.level is None or user.profile.level < user_level:
                if user.profile.level < user_level:
                    #~ print 20120715, user.profile.level
                    return False
                return allow_user_level(action,user,obj,state)
                
        if owner is not None:
            allow_owner = allow
            def allow(action,user,obj,state):
                if obj is not None and (user == obj.user) != owner:
                    return False
                return allow_owner(action,user,obj,state)
                
        if user_groups is not None:
            if isinstance(user_groups,basestring):
                user_groups = user_groups.split()
            if user_level is None:
                user_level = UserLevels.user
                #~ raise Exception("20120621")
            for g in user_groups:
                UserGroups.get_by_value(g) # raise Exception if no such group exists
                #~ if not UserGroups.get_by_name(g):
                    #~ raise Exception("Invalid UserGroup %r" % g)
            allow1 = allow
            def allow(action,user,obj,state):
                if not allow1(action,user,obj,state): return False
                for g in user_groups:
                    level = getattr(user.profile,g+'_level')
                    if level >= user_level:
                        return True
                #~ if isinstance(elem,Permittable):
                    #~ print '20120630 not for you:', elem
                return False
                
        if states is not None:
            if not isinstance(actor.workflow_state_field,choicelists.ChoiceListField):
                raise Exception(
                    """\
Cannot specify `states` when `workflow_state_field` is %r.
                    """ % actor.workflow_state_field)
            #~ else:
                #~ print 20120621, "ok", actor
            lst = actor.workflow_state_field.choicelist
            #~ states = frozenset([getattr(lst,n) for n in states])
            #~ possible_states = [st.name for st in lst.items()] + [BLANK_STATE]
            ns = []
            for n in states:
                ns.append(lst.get_by_name(n))
                #~ if n:
                    #~ ns.append(getattr(lst,n))
                #~ else:
                    #~ ns.append(lst.blank_item)
                    
                #~ if not st in possible_states:
                    #~ raise Exception("Invalid state %r, must be one of %r" % (st,possible_states))
            states = frozenset(ns)
            allow2 = allow
            def allow(action,user,obj,state):
                if not allow2(action,user,obj,state): return False
                if obj is None: return True
                return state in states
        #~ return perms.Permission(allow)
        
        if not readonly:
            allow3 = allow
            def allow(action,user,obj,state):
                if not allow3(action,user,obj,state): return False
                if user.profile.readonly:
                    #~ if isinstance(elem,Permittable):
                        #~ print '20120630 not readonly:', elem
                    return False
                return True
            
        
        
        if debug_permissions: # False:
            allow4 = allow
            def allow(action,user,obj,state):
                v = allow4(action,user,obj,state)
                if not v:
                    logger.info(u"debug_permissions %s %s.required(%s,%s,%s), allow(%s,%s,%s)--> %s",
                      actor,action.name,user_level,user_groups,states,user.username,obj2str(obj),state,v)
                return v
        return allow
    except Exception,e:
        raise Exception("Exception while making permissions for %s: %s" % (actor,e))

        



## Copyright 2009-2012 Luc Saffre
## This file is part of the Lino project.
## Lino is free software; you can redistribute it and/or modify 
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## Lino is distributed in the hope that it will be useful, 
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with Lino; if not, see <http://www.gnu.org/licenses/>.

import logging
logger = logging.getLogger(__name__)

import traceback
#~ from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_unicode
from django.conf import settings
from django import http


import lino
from lino.utils import AttrDict
from lino.utils import curry
from lino.utils import babel
from lino.utils import Warning

from lino.ui import requests as ext_requests

from lino.core.modeltools import resolve_model

#~ from lino.core.perms import UserLevels
#~ from lino.core import perms 


class VirtualRow(object):
    def __init__(self,**kw):
        self.update(**kw)
        
    def update(self,**kw):
        for k,v in kw.items():
            setattr(self,k,v)

    def get_row_permission(self,user,state,action):
        if action.readonly:
            return True
        return False 
            

class PhantomRow(VirtualRow):
    def __init__(self,request,**kw):
        self._ar = request
        VirtualRow.__init__(self,**kw)
        
    def __unicode__(self):
        return unicode(self._ar.get_action_title())
        
class EmptyTableRow(VirtualRow):
    """
    Base class for virtual rows of an :class:`EmptyTable`.
    """
    def __init__(self,table,**kw):
        self._table = table
        VirtualRow.__init__(self,**kw)
        
    def __unicode__(self):
        return unicode(self._table.label)
        
    def get_print_language(self,pm):
        return babel.DEFAULT_LANGUAGE
        
    def get_templates_group(self):
        return self._table.app_label + '/' + self._table.__name__
    
    def filename_root(self):
        return self._table.app_label + '.' + self._table.__name__









class Hotkey:
    keycode = None
    shift = False
    ctrl = False
    alt = False
    inheritable = ('keycode','shift','ctrl','alt')
    def __init__(self,**kw):
        for k,v in kw.items():
            setattr(self,k,v)
            
    def __call__(self,**kw):
        for n in self.inheritable:
            if not kw.has_key(n):
                kw[n] = getattr(self,n)
            return Hotkey(**kw)
      
# ExtJS src/core/EventManager-more.js
RETURN = Hotkey(keycode=13)
ESCAPE = Hotkey(keycode=27)
PAGE_UP  = Hotkey(keycode=33)
PAGE_DOWN = Hotkey(keycode=34)
INSERT = Hotkey(keycode=44)
DELETE = Hotkey(keycode=46)
    
    
class ConfirmationRequired(Exception):
    """
    This is the special exception risen when an Action calls 
    :meth:`ActionRequest.confirm`.
    """
    def __init__(self,step,messages):
        self.step = step
        self.messages = messages
        Exception.__init__(self)


class Action(object):
    """
    Abstract base class for all Actions
    """
    
    sort_index = 90
    """
    
    Predefined sort_index values are:
    
    ===== =================================
    value action
    ===== =================================
    20    :class:`Insert <InsertRow>`
    30    :class:`Delete <DeleteSelected>`
    50    :class:`Print <lino.mixins.printable.BasePrintAction>`
    60    :attr:`Duplicate <lino.mixins.duplicable.Duplicable.duplicate_row>`
    90    default for all custom row actions created using :func:`@dd.action <action>`
    ===== =================================
    
    """
    
    label = None
    """
    The text to appear on the button.
    """
    
    help_text = None
    """
    A help text that shortly explains what this action does.
    ExtJS uses this as tooltip text.
    """
    
    auto_save = True
    """
    What to do when this action is being called while the user is on a dirty record.
    
    - `False` means: forget any changes in current record and run the action.
    - `True` means: save any changes in current record before running the action.
    - `None` means: ask the user.
    """
    
    required = {}
    """
    A dict with permission requirements.
    See :func:`lino.core.perms.make_permission_handler`.
    """
    
    name = None
    """
    Internally used to store the name of this action within the Actor's namespace.
    """
    
    url_action_name = None
    """
    """
    
    actor = None
    """
    Internally used to store the :class:`lino.core.actors.Actor` 
    who owns this action.
    """
    
    key = None
    """
    The hotkey. Currently not used.
    """
    
    callable_from = None
    """
    Either `None`(default) or a tuple of class 
    objects (subclasses of :class:`Action`).
    If specified, this action is available only within a window 
    that has been opened by one of the given actions.
    """
    
    default_format = 'html'
    """
    Used internally.
    """
    
    readonly = True
    """
    Whether this action possibly modifies data *in the given object*.
    This means that :class:`InsertRow` is a `readonly` action.
    """
    
    opens_a_window = False
    """
    Used internally to say whether this action opens a window.
    """
    
    hide_top_toolbar = False
    """
    Used internally if :attr:`opens_a_window` to say whether the window has a top toolbar.
    """
    
    hide_navigator = False
    """
    Used internally if :attr:`opens_a_window` to say whether the window has a navigator.
    """
    
    show_in_bbar = True
    """
    Used internally.
    Whether this action should be displayed as a button in the bottom toolbar and the context menu.
    """
    
    show_in_workflow = False
    """
    Used internally.
    Whether this action should be displayed 
    as the :meth:`workflow_buttons <lino.core.actors.Actor.workflow_buttons>`.
    """
    
    custom_handler = False
    """
    Whether this action is implemented as Javascript function call.
    (...)
    """
    
    
    #~ def __init__(self,label=None,url_action_name=None,required={},**kw):
    def __init__(self,label=None,**kw):
        """
        The first argument is the optional `label`,
        other arguments should be specified as keywords and can be 
        any of the existing class attributes.
        """
        #~ if url_action_name is not None:
            #~ if not isinstance(url_action_name,basestring):
                #~ raise Exception("%s name %r is not a string" % (self.__class__,url_action_name))
            #~ self.url_action_name = url_action_name
        if label is not None:
            self.label = label
            
        #~ if label is None:
            #~ label = self.label or self.url_action_name 
        for k,v in kw.items():
            if not hasattr(self,k):
                raise Exception("Invalid keyword %s" % k)
            setattr(self,k,v)
        self.set_required()
        #~ self.set_required(**required)
        assert self.callable_from is None or isinstance(
            self.callable_from,(tuple,type)), "%s" % self

        
    def set_required(self,**kw):
        """
        Override existing permission requirements.
        Arguments: see :func:`lino.core.perms.make_permission_handler`.
        """
        #~ logger.info("20120628 set_required %s(%s)",self,kw)
        new = dict()
        new.update(self.required)
        new.update(kw)
        self.required = new
        #~ if isinstance(self,StateAction):
        if self.required.has_key('states'):
            self.show_in_workflow = True
            self.custom_handler = True
            self.show_in_bbar = False
        else:
            self.show_in_workflow = False
            self.show_in_bbar = True
        
    def __str__(self):
        #~ raise Exception("Must use action2str(actor,action)")
        if self.actor is None:
            #~ raise Exception("tried to call str() on general action %s" % self.name)
            return repr(self)
            #~ raise Exception("Tried to call str() on shared action %r" % self)
        if self.name is None:
            return repr(self)
        return str(self.actor) + '.' + self.name
        
    #~ def set_permissions(self,*args,**kw)
        #~ self.permission = perms.factory(*args,**kw)
        
    def attach_to_actor(self,actor,name):
        if self.name is not None:
            raise Exception("%s tried to attach named action %s" % (actor,self))
        if self.actor is not None:
            raise Exception("%s tried to attach action %s of %s" % (actor,name,self.actor))
        self.name = name
        self.actor = actor
        if self.label is None:
            self.label = name
        if actor.hide_top_toolbar:
            self.hide_top_toolbar = True
        if self.help_text is None and self is actor.default_action:
            self.help_text  = actor.help_text
        #~ if name == 'default_action':
            #~ print 20120527, self
            
    #~ def contribute_to_class(self,model,name):
        #~ ma = model.__dict__.get('_lino_model_actions',None)
        #~ if ma is None:
            #~ ma = dict()
            #~ model._lino_model_actions = ma
            #~ # model.__dict__.set('_lino_model_actions',ma)
        #~ ma[name] = self
        #~ self.name = name
        #~ # model.__dict__['_lino_model_actions'] = d
        
    def __unicode__(self):
        return force_unicode(self.label)
        
    #~ def get_view_permission(self,user):
        #~ """
        #~ E.g. DispatchAction is not available for a User with empty partner
        #~ """
        #~ return True
        
    def get_button_label(self):
        if self.actor is None:
            return self.label 
        if self is self.actor.default_action:
            return self.label 
        else:
            return u"%s %s" % (self.label,self.actor.label)
            
    def get_action_permission(self,user,obj,state):
        """
        The default implementation simply calls this action's 
        permission handler.
        Derived Action classes may override this to add vetos.
        E.g. DispatchAction is not available for a User with empty partner.
        """
        return self.allow(user,obj,state)
        
    #~ def run(self,elem,ar,**kw):
        #~ raise NotImplementedError("%s has no run() method" % self.__class__)


class TableAction(Action):
    """
    TODO: get_action_permission and required_states 
    are needed here because `disabled_actions` also asks InsertRow whether 
    it's permitted on that row. It's in fact not correct to ask this for 
    the Insert button. Has to do with the fact that the Insert button is 
    in the bottom toolbar though it should be in the top toolbar...
    """
  
    #~ required_states = None
    
    def get_action_title(self,rr):
        return rr.get_title()
        

class RowAction(Action):
    """
    Base class for actions that are executed on an individual row.
    """
    
    def run(self,row,ar,**kw):
        """
        Execute the action on the given `row`. `ar` is an :class:`ActionRequest` 
        object representing the context where the action is running.
        """
        raise NotImplementedError("%s has no run() method" % self.__class__)

    #~ def get_action_permission(self,user,obj):
        #~ return self.actor.get_row_permission(self,user,obj)
            
    def attach_to_actor(self,actor,name):
        super(RowAction,self).attach_to_actor(actor,name)
        if not self.url_action_name:
            self.url_action_name = name 


class StateAction(RowAction):
    ajax = True
    def __init__(self,actor,target_state,**kw):
        self.target_state = target_state
        kw.update(label=getattr(target_state,'action_label',target_state.text))
        required = getattr(target_state,'required',None)
        if required is not None:
            if target_state.name:
                m = getattr(actor.model,'allow_state_'+target_state.name,None)
                if m is not None:
                    def allow(action,user,obj,state):
                        return m(obj,user)
                    required.update(allow=allow)
            kw.update(required=required)
        help_text = getattr(target_state,'help_text',None)
        if help_text:
            kw.update(help_text=help_text)
        super(StateAction,self).__init__(**kw)
        #~ print 20120709, self, self.show_in_workflow
        
    def run(self,row,ar,**kw):
        state_field_name = self.actor.workflow_state_field.attname
        assert isinstance(state_field_name,basestring)
        #~ old = row.state
        old = getattr(row,state_field_name)
        row.before_state_change(ar,kw,old,self.target_state)
        #~ row.state = self.target_state
        setattr(row,state_field_name,self.target_state)
        row.save()
        row.after_state_change(ar,kw,old,self.target_state)
        return ar.ui.success_response(**kw)
        
    

class RedirectAction(Action):
    
    def get_target_url(self,elem):
        raise NotImplementedError
        



class GridEdit(TableAction):
  
    show_in_workflow = False
    opens_a_window = True

    callable_from = tuple()
    url_action_name = 'grid'
    
    def attach_to_actor(self,actor,name):
        #~ self.label = actor.button_label or actor.label
        self.label = actor.label
        super(GridEdit,self).attach_to_actor(actor,name)

    def get_window_layout(self):
        #~ return self.actor.list_layout
        return None


class ShowDetailAction(RowAction):
    """
    An action that opens the Detail Window of its actor.
    """
    opens_a_window = True
    show_in_workflow = False
    
    #~ sort_index = 1
    callable_from = (GridEdit,)
    #~ show_in_detail = False
    #~ needs_selection = True
    url_action_name = 'detail'
    label = _("Detail")
    
    def get_window_layout(self):
        return self.actor.detail_layout
        
    #~ def get_elem_title(self,elem):
        #~ return _("%s (Detail)")  % unicode(elem)
        

RowAction.callable_from = (GridEdit,ShowDetailAction)

class InsertRow(TableAction):
    """
    Opens the Insert window filled with a blank row. 
    The new row will be actually created only when this 
    window gets submitted.
    """
    label = _("New")
    show_in_workflow = False
    opens_a_window = True
    hide_navigator = True
    sort_index = 20
    hide_top_toolbar = True
    #~ readonly = False # see blog/2012/0726
    required = dict(user_level='user')
    callable_from = (GridEdit,ShowDetailAction)
    url_action_name = 'insert'
    #~ label = _("Insert")
    key = INSERT # (ctrl=True)
    #~ needs_selection = False
    
    def get_action_title(self,rr):
        return _("Insert into %s") % force_unicode(rr.get_title())
        
    def get_window_layout(self):
        return self.actor.insert_layout or self.actor.detail_layout

    def get_action_permission(self,user,obj,state):
        # see blog/2012/0726
        if user.profile.readonly: 
            return False
        return super(InsertRow,self).get_action_permission(user,obj,state)




class DuplicateRow(RowAction):
    opens_a_window = True
  
    readonly = False
    required = dict(user_level='user')
    callable_from = (GridEdit,ShowDetailAction)
    url_action_name = 'duplicate'
    label = _("Duplicate")


class ShowEmptyTable(ShowDetailAction):
    callable_from = tuple()
    url_action_name = 'show' 
    default_format = 'html'
    #~ hide_top_toolbar = True
    hide_navigator = True
    
    def attach_to_actor(self,actor,name):
        self.label = actor.label
        ShowDetailAction.attach_to_actor(self,actor,name)
        #~ print 20120523, actor, name, 'setup', unicode(self.label)
        
    def get_action_title(self,rr):
        return rr.get_title()
    #~ def __str__(self):
        #~ return str(self.actor)+'.'+self.name
        
    

class UpdateRowAction(RowAction):
    show_in_workflow = False
    readonly = False
    required = dict(user_level='user')
    

class ListAction(Action):
    """
    Base class for actions that are executed server-side on an individual row.
    """
    callable_from = (GridEdit,)
    

class DeleteSelected(RowAction):
    """
    Delete the row.
    """
    auto_save = False
    sort_index = 30
    readonly = False
    show_in_workflow = False
    required = dict(user_level='user')
    callable_from = (GridEdit,ShowDetailAction)
    #~ needs_selection = True
    label = _("Delete")
    url_action_name = 'delete'
    key = DELETE # (ctrl=True)
    #~ client_side = True
    
        
class SubmitDetail(RowAction):
    label = _("Save")
    auto_save = False
    show_in_workflow = False
    #~ show_in_bbar = True
    readonly = False
    required = dict(user_level='user')
    #~ url_action_name = 'SubmitDetail'
    callable_from = (ShowDetailAction,)
    
class SubmitInsert(SubmitDetail):
    #~ url_action_name = 'SubmitInsert'
    label = _("Create")
    #~ label = _("Insert")
    callable_from = (InsertRow,)




class ActionRequest(object):
    """
    Holds information about an indivitual web request and provides methods like

    - :meth:`get_user <lino.core.actions.ActionRequest.get_user>`
    - :meth:`confirm <lino.core.actions.ActionRequest.confirm>`
    - :meth:`success_response <lino.ui.base.UI.success_response>`
    - :meth:`error_response <lino.ui.base.UI.error_response>`
    - :meth:`spawn <lino.core.actions.ActionRequest.spawn>`

    
    """
    create_kw = None
    renderer = None
    
    def __init__(self,ui,actor,request=None,action=None,renderer=None,**kw):
        #~ ActionRequest.__init__(self,ui,action)
        if ui is None:
            ui = settings.LINO.ui
            #~ from lino.ui.extjs3 import ui
        self.ui = ui
        self.error_response = ui.error_response
        self.success_response = ui.success_response
        self.renderer = renderer
        self.action = action or actor.default_action
        self.step = 0 # confirmation counter
        #~ self.report = actor
        self.actor = actor
        self.request = request
        if request is not None:
            if request.method == 'PUT':
                rqdata = http.QueryDict(request.raw_post_data)
            else:
                rqdata = request.REQUEST
            kw = self.parse_req(request,rqdata,**kw)
        #~ 20120605 self.ah = actor.get_handle(ui)
        self.setup(**kw)
        # 20120605 : Actor.get_request_handle() ar instead of ui
        self.ah = actor.get_request_handle(self)
        
        if self.actor.parameters and request is not None:
            param_values = self.ui.parse_params(self.ah,request)
                
            for k,pf in self.actor.parameters.items():
                v = param_values.get(k,None)
                if v is None:
                    v = pf.get_default()
                self.param_values.define(k,v)
            if param_values:
                #~ logger.info("20120608 param_values is %s",param_values)
                for k,v in param_values.items():
                    self.param_values.define(k,v)
                
        
    def href_to(self,*args,**kw):
        return self.renderer.href_to(self,*args,**kw)
        
    def parse_req(self,request,rqdata,**kw): 
        #~ if self.actor.parameters:
            #~ kw.update(param_values=self.ui.parse_params(self.ah,request))
        kw.update(user=request.user)
        kw.update(subst_user=request.subst_user)
        kw.update(requesting_panel=request.requesting_panel)
        #~ if settings.LINO.user_model:
            #~ username = rqdata.get(ext_requests.URL_PARAM_SUBST_USER,None)
            #~ if username:
                #~ try:
                    #~ kw.update(subst_user=settings.LINO.user_model.objects.get(username=username))
                #~ except settings.LINO.user_model.DoesNotExist, e:
                    #~ pass
        #~ logger.info("20120723 ActionRequest.parse_req() --> %s",kw)
        return kw
      
    def setup(self,
            user=None,
            subst_user=None,
            param_values={},
            known_values=None,
            requesting_panel=None,
            renderer=None,
            **kw):
        self.requesting_panel = requesting_panel
        self.user = user
        if renderer is not None:
            self.renderer = renderer
        if self.actor.parameters:
            self.param_values = AttrDict(**param_values)
            #~ self.param_values = param_values
        self.subst_user = subst_user
        #~ 20120111 
        #~ self.known_values = known_values or self.report.known_values
        #~ if self.report.known_values:
        #~ d = dict(self.report.known_values)
        for k,v in self.actor.known_values.items():
            kw.setdefault(k,v)
        if known_values:
            kw.update(known_values)
        self.known_values = kw
        
        
    def confirm(self,*messages):
        """
        Calling this from an Action's :meth:`Action.run` method will
        interrupt the execution, send the specified message(s) back to 
        the user, waiting for confirmation before continuing.
        
        Note that this is implemented without any server sessions 
        and cookies. While this system is genial, it has one drawback 
        which you should be aware of: the code execution does not 
        *continue* after the call to `confirm` but starts again at the 
        beginning (with the difference that the client this time calls it with 
        an internal `step` parameter that tells Lino that this `confirm()` 
        has been answered and should no longer raise stop execution.
        """
        assert len(messages) > 0 and messages[0], "At least one non-empty message required"
        self.step += 1
        if int(self.request.REQUEST.get(ext_requests.URL_PARAM_ACTION_STEP,'0')) >= self.step:
            return
        raise ConfirmationRequired(self.step,messages)
        
    def create_phantom_row(self,**kw):
        if self.create_kw is None or not self.actor.editable:
            #~ logger.info('20120519 %s.create_phantom_row(), %r', self,self.create_kw)
            return 
        #~ if not self.actor.get_permission(self.get_user(),self.actor.create_action):
        #~ if not self.actor.allow_create(self.get_user(),None,None):
        if self.actor.create_action is not None:
            if not self.actor.create_action.allow(self.get_user(),None,None):
                return
        return PhantomRow(self,**kw)
      
    def create_instance(self,**kw):
        if self.create_kw:
            kw.update(self.create_kw)
        #logger.debug('%s.create_instance(%r)',self,kw)
        if self.known_values:
            kw.update(self.known_values)
        #~ print "20120527 create_instance", self, kw
        obj = self.actor.create_instance(self,**kw)
        #~ print 20120630, self.actor, 'actions.TableRequest.create_instance'
        #~ if self.known_values is not None:
            #~ self.ah.store.form2obj(self.known_values,obj,True)
            #~ for k,v in self.known_values:
                #~ field = self.model._meta.get_field(k) ...hier
                #~ kw[k] = v
        return obj
        
    def get_data_iterator(self):
        raise NotImplementedError
        
    def get_base_filename(self):
        return str(self.actor)
        #~ s = self.get_title()
        #~ return s.encode('us-ascii','replace')
        
    def get_user(self):
        """
        Return the :class:`User <lino.modlib.users.models.User>` 
        instance of the user who issued the request.
        If the authenticated user is acting as somebody else, 
        return that :class:`User <lino.modlib.users.models.User>` instance.
        """
        return self.subst_user or self.user
        
    def get_action_title(self):
        return self.action.get_action_title(self)
        
    def get_title(self):
        return self.actor.get_title(self)
        
    def render_to_dict(self):
        return self.action.render_to_dict(self)
        
    def get_request_url(self,*args,**kw):
        return self.ui.get_request_url(self,*args,**kw)

    def get_status(self,ui,**kw):
        if self.actor.parameters:
            kw.update(param_values=self.ah.store.pv2dict(ui,self.param_values))
        bp = kw.setdefault('base_params',{})
        if self.subst_user is not None:
            #~ bp[ext_requests.URL_PARAM_SUBST_USER] = self.subst_user.username
            bp[ext_requests.URL_PARAM_SUBST_USER] = self.subst_user.id
        return kw
        

    def spawn(self,actor=None,**kw):
        """
        Create a new ActionRequest, taking default values from this one.
        """
        kw.setdefault('user',self.user)
        kw.setdefault('subst_user',self.subst_user)
        kw.setdefault('requesting_panel',self.requesting_panel)
        kw.setdefault('renderer',self.renderer)
        #~ kw.setdefault('request',self.request) 
        # removed 20120702 because i don't want to inherit quick_search from spawning request
        # and because i couldn't remember why 'request' was passed to the spawned request.
        if actor is None:
            actor = self.actor
        return self.ui.request(actor,**kw)
        
    def absolute_uri(self,*args,**kw):
        ar = self.spawn(*args,**kw)
        location = ar.renderer.get_request_url(ar)
        return self.request.build_absolute_uri(location)
        

def action(*args,**kw):
    """
    Decorator to define custom actions.
    Same signature as :meth:`Action.__init__`.
    In practice you'll possibly use:
    :attr:`label <Action.label>`,
    :attr:`help_text <Action.help_text>` and
    :attr:`required <Action.required>`
    """
    def decorator(fn):
        kw.setdefault('custom_handler',True)
        a = RowAction(*args,**kw)
        #~ a.run = curry(fn,a)
        a.run = fn
        return a
    return decorator
    
#~ def action2str(actor,action):
    #~ return str(actor) + '.' + action.name



## Copyright 2010-2012 Luc Saffre
## This file is part of the Lino project.
## Lino is free software; you can redistribute it and/or modify 
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## Lino is distributed in the hope that it will be useful, 
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with Lino; if not, see <http://www.gnu.org/licenses/>.

"""

"""

import logging
logger = logging.getLogger(__name__)


import datetime

from django.db import models
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
#~ from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import string_concat

from lino import dd
#~ from lino.models import Workflow
#~ from lino.utils import perms
from lino.core.modeltools import full_model_name
from lino.core import frames
from lino.core import actions
from lino.utils.choosers import chooser
from lino.mixins.duplicable import Duplicable
    
from lino.core.perms import UserLevels

#~ class Owned(dd.Model):
class Controllable(dd.Model):
    """
    Mixin for models that are "controllable" by another database object.
    
    Defines three fields `owned_type`, `owned_id` and `owned`.
    And a class attribute :attr:`owner_label`.
    
    For example in :mod:`lino.modlibs.cal`, the owner of a Task or Event 
    is some other database object that caused the task's or event's 
    creation.
    
    Or in :mod:`lino.modlib.sales` and :mod:`lino.modlib.purchases`,
    an invoice may cause one or several Tasks 
    to be automatically generated when a certain payment mode 
    is specified.
    
    Controllable objects are "governed" or "controlled"
    by their controller:
    If the controller gets modified, it may decide to delete or 
    modify some or all of her controlled objects.
    
    Non-automatic tasks always have an empty `controller` field.
    Some fields are read-only on an automatic Task because 
    it makes no sense to modify them.
    
    
    
    """
    # Translators: will also be concatenated with '(type)' '(object)'
    owner_label = _('Controlled by')
    """
    The labels (`verbose_name`) of the fields 
    `owned_type`, `owned_id` and `owned`
    are derived from this attribute which 
    may be overridden by subclasses.
    
    """
    
    class Meta:
        abstract = True
        
    owner_type = models.ForeignKey(ContentType,
        editable=True,
        blank=True,null=True,
        verbose_name=string_concat(owner_label,' ',_('(type)')))
    owner_id = dd.GenericForeignKeyIdField(
        owner_type,
        editable=True,
        blank=True,null=True,
        verbose_name=string_concat(owner_label,' ',_('(object)')))
    owner = dd.GenericForeignKey(
        'owner_type', 'owner_id',
        verbose_name=owner_label)
        
    #~ owner_panel= dd.FieldSet(_("Owner"),
        #~ "owner_type owner_id",
        #~ owner_type=_("Model"),
        #~ owner_id=_("Instance"))
    
    
    @chooser(instance_values=True)
    def owner_id_choices(cls,owner_type):
        if owner_type:
            return owner_type.model_class().objects.all()
        return []
      
    #~ owner_id_choices.instance_values = True
    #~ owner_id_choices = classmethod(owner_id_choices)
        
    def get_owner_id_display(self,value):
        if self.owner_type:
            try:
                return unicode(self.owner_type.get_object_for_this_type(pk=value))
            except self.owner_type.model_class().DoesNotExist,e:
                return "%s with pk %r does not exist" % (
                    full_model_name(self.owner_type.model_class()),value)
            
            
    def update_owned_instance(self,controllable):
        """
        If this (acting as a controller) is itself controlled, 
        forward the call to the controller.
        """
        if self.owner:
            self.owner.update_owned_instance(controllable)
        #~ m = getattr(self.owner,'update_owned_instance',None)
        #~ if m:
            #~ m(controllable)

    #~ def data_control(self):
        #~ "Used by :class:`lino.models.DataControlListing`."
        #~ msgs = []
        #~ ct = ContentType.objects.get_for_id()
        #~ ...
        #~ msgs.append(unicode(e))
        #~ return msgs
        
    #~ def is_user_modified(self):
        #~ """
        #~ When a controlled `cal.Event` has been manually modified 
        #~ by the user, then the controller is no longer
        #~ """
        #~ raise NotImplementedError
        #~ return False

    def save(self,*args,**kw):
        if self.owner: #  and not self.is_user_modified():
            self.owner.update_owned_instance(self)
            #~ print "20120627 called update_owned_instance of ", self.owner.__class__
            #~ m = getattr(self.owner,'update_owned_instance',None)
            #~ if m:
                #~ m(self)
        super(Controllable,self).save(*args,**kw)
        if self.owner: #  and self.is_user_modified():
            self.owner.after_update_owned_instance(self)
            #~ m = getattr(self.owner,'after_update_owned_instance',None)
            #~ if m:
                #~ m(self)



#~ class AutoUser(dd.Model):
class UserAuthored(dd.Model):
    """
    Mixin for models that have a `user` field which is automatically 
    set to the requesting user.
    Also defines a `ByUser` base table which fills the master instance 
    from the web request.
    """
    class Meta:
        abstract = True
        
    #~ user = models.ForeignKey('contacts.Partner',
        #~ verbose_name=_("user"),
        #~ related_name="%(app_label)s_%(class)s_set_by_user",
        #~ blank=True,null=True
        #~ )
            
    if settings.LINO.user_model: 
      
        workflow_owner_field = 'user' # used by :mod:`lino.modlib.workflows.models`
        
        user = models.ForeignKey(settings.LINO.user_model,
            verbose_name=_("Author"),
            related_name="%(app_label)s_%(class)s_set_by_user",
            blank=True,null=True
            )
        
    #~ def on_duplicate(self,ar):
        #~ self.user = ar.get_user()
        #~ super(AutoUser,self).on_duplicate(ar)
        
        
    def on_create(self,ar):
        """
        Adds the requesting user to the `user` field.
        """
        if self.user_id is None:
            u = ar.get_user()
            if u is not None:
                self.user = u
        super(UserAuthored,self).on_create(ar)
        
    #~ def update_owned_instance(self,other):
        #~ print '20120627 AutoUser.update_owned_instance'
        #~ other.user = self.user
        #~ super(UserAuthored,self).update_owned_instance(other)
        

    def get_row_permission(self,user,state,action):
        """
        Only system managers can edit other users' work.
        """
        if self.user != user and user.profile.level < UserLevels.manager:
            return False
        if not super(UserAuthored,self).get_row_permission(user,state,action):
            return False
        return True

AutoUser = UserAuthored # backwards compatibility


if settings.LINO.user_model: 
  
    class ByUser(dd.Table):
        master_key = 'user'
        #~ can_view = perms.is_authenticated
        
        @classmethod
        def get_actor_label(self):
            return string_concat(
                _("My "),self.model._meta.verbose_name_plural)
            #~ return _("My %s") % self.model._meta.verbose_name_plural
            
        @classmethod
        def setup_request(self,rr):
            #~ logger.info("mixins.ByUser.setup_request")
            if rr.master_instance is None:
                rr.master_instance = rr.get_user()
                
                
else:
  
    # dummy Table for userless sites
    class ByUser(dd.Table): pass 
  



class CreatedModified(dd.Model):
    """
    Adds two timestamp fields `created` and `modified`.    
    
    We don't use Djangos auto_now and auto_now_add features because:
    
    - 20110829 the modified field did not get updated after save()
      didn't investigate further since the workaround shown at
      http://stackoverflow.com/questions/1737017/django-auto-now-and-auto-now-add
      is ok for me.
      
    - :doc:`/blog/2011/0901`
    
    """
    class Meta:
        abstract = True
        
    created = models.DateTimeField(editable=False)
    modified = models.DateTimeField(editable=False)
    
    def save(self, *args, **kwargs):
        '''
        On save, update timestamps.
        '''
        if not settings.LINO.loading_from_dump:
            if not self.pk:
                self.created = datetime.datetime.now()
            self.modified = datetime.datetime.now()
        super(CreatedModified, self).save(*args, **kwargs)


class Sequenced(Duplicable):
    """
    Abstract base class for models that have a sequence number `seqno`.
    """
  
    class Meta:
        abstract = True
        ordering = ['seqno']
        
    seqno = models.IntegerField(
        blank=True,null=False,
        verbose_name=_("Seq.No."))
        
        
    @dd.action(_("Duplicate"))
    def duplicate_row(self,ar):
        #~ print '20120605 duplicate_row', self.seqno, self.account
        seqno = self.seqno
        qs = self.get_siblings().filter(seqno__gte=seqno).reverse()
        for s in qs:
            #~ print '20120605 duplicate_row inc', s.seqno, s.account
            s.seqno += 1
            s.save()
        return super(Sequenced,self).duplicate_row.run(self,ar,seqno=seqno)
        
    def __unicode__(self):
        return unicode(_("Row # %s") % self.seqno)
        
    
    def get_siblings(self):
        """
        The default implementation sets a global sequencing
        by returning all objects of this model.
        Overridden in :class:`lino.modlib.thirds.models.Third`.
        """
        return self.__class__.objects.order_by('seqno')      
        
    def set_seqno(self):
        """
        Initialize `seqno` to the `seqno` of eldest sibling + 1.
        """
        qs = self.get_siblings()
        n = qs.count()
        if n == 0:
            self.seqno = 1
        else:
            last = qs[n-1]
            self.seqno = last.seqno + 1
        
    
    def full_clean(self,*args,**kw):
        if self.seqno is None:
            self.set_seqno()
        super(Sequenced,self).full_clean(*args,**kw)
  

class ProjectRelated(dd.Model):
    """
    Mixin for Models that are automatically 
    related to a "project". 
    A project means here 
    "the central most important thing that is used to classify most other things".
    For example in lino.apps.pcsw, the "project" is a Client.
    
    Whether an application has such a concept of "project", 
    and which model has this privileged status, 
    is set in :attr:`lino.Lino.project_model`.
    
    """
    
    class Meta:
        abstract = True
        
    if settings.LINO.project_model:
        project = models.ForeignKey(
            settings.LINO.project_model,
            blank=True,null=True,
            related_name="%(app_label)s_%(class)s_set_by_project",
            )

    #~ def summary_row(self,ui,rr,**kw):
    def summary_row(self,ar,**kw):
        s = ar.href_to(self)
        #~ s = ui.ext_renderer.href_to(self)
        if settings.LINO.project_model:
            #~ if self.project and not dd.has_fk(rr,'project'):
            if self.project:
                #~ s += " (" + ui.href_to(self.project) + ")"
                s += " (" + ar.href_to(self.project) + ")"
        return s
            
    def update_owned_instance(self,other):
        """
        When a :class:`project-related <ProjectRelated>` 
        object controls another project-related object, 
        then the controlled automatically inherits 
        the `project` of its controller.
        """
        if isinstance(other,ProjectRelated):
            other.project = self.project
        super(ProjectRelated,self).update_owned_instance(other)
        
    def get_mailable_recipients(self):
        if isinstance(self.project,settings.LINO.modules.contacts.Partner):
            if self.project.email:
                yield ('to',self.project)
        for r in super(ProjectRelated,self).get_mailable_recipients():
            yield r

    def get_postable_recipients(self):
        if isinstance(self.project,settings.LINO.modules.contacts.Partner):
            yield self.project
        for p in super(ProjectRelated,self).get_postable_recipients():
            yield p


from lino.mixins.printable import Printable, PrintableType, CachedPrintable, TypedPrintable, DirectPrintAction
from lino.mixins.uploadable import Uploadable
#~ from lino.mixins.mails import Recipient, Mail
from lino.utils.dblogger import DiffingMixin
#~ from lino.mixins.personal import SexField, PersonMixin

from lino.core import actions
from lino.mixins import printable


class EmptyTable(frames.Frame):
    """
    A "Table" that has exactly one virtual row and thus is visible 
    only using a Detail view on that row.
    """
    #~ has_navigator = False
    #~ hide_top_toolbar = True
    hide_navigator = True
    default_list_action_name = 'show'
    default_elem_action_name =  'show'
    default_action = actions.ShowEmptyTable()
    do_print = DirectPrintAction()
    
    #~ @classmethod
    #~ def do_setup(self):
        #~ # logger.info("%s.__init__()",self.__class__)
        #~ # if not self.__class__ is Frame:
        #~ if self is not EmptyTable:
            #~ # assert self.default_action_class is None
            #~ # if self.label is None:
                #~ # raise Exception("%r has no label" % self)
            #~ # self.default_action = actions.ShowEmptyTable()
            #~ # self.default_action = self.add_action(actions.ShowEmptyTable())
            #~ super(Frame,self).do_setup()
            #~ # self.setup_actions()
            #~ # self.add_action(self.default_action)

    #~ @classmethod
    #~ def setup_actions(self):
        #~ super(EmptyTable,self).setup_actions()
        #~ from lino.mixins.printable import DirectPrintAction
        #~ self.add_action(DirectPrintAction())
        
            
    @classmethod
    def create_instance(self,req,**kw):
        #~ if self.known_values:
            #~ kw.update(self.known_values)
        if self.parameters:
            kw.update(req.param_values)

        #~ for k,v in req.param_values.items():
            #~ kw[k] = v
        #~ for k,f in self.parameters.items():
            #~ kw[k] = f.value_from_object(None)
        obj = actions.EmptyTableRow(self,**kw)
        kw = req.ah.store.row2dict(req,obj)
        obj._data = kw
        obj.update(**kw)
        return obj
    
    #~ @classmethod
    #~ def elem_filename_root(self,elem):
        #~ return self.app_label + '.' + self.__name__

    @classmethod
    def get_data_elem(self,name):
        de = super(EmptyTable,self).get_data_elem(name)
        if de is not None:
            return de
        a = name.split('.')
        if len(a) == 2:
            return getattr(getattr(settings.LINO.modules,a[0]),a[1])
            


#~ from lino.models import Workflowable


# -*- coding: UTF-8 -*-
## Copyright 2012 Luc Saffre
## This file is part of the Lino project.
## Lino is free software; you can redistribute it and/or modify 
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## Lino is distributed in the hope that it will be useful, 
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with Lino; if not, see <http://www.gnu.org/licenses/>.

"""
Fills the Sectors table using the official data from 
http://www.bcss.fgov.be/binaries/documentation/fr/documentation/general/lijst_van_sectoren_liste_des_secteurs.xls    

"""

from django.conf import settings
from lino.utils import ucsv
#~ from lino.utils.babel import babel_values
from lino.utils import babel
from lino.core.modeltools import resolve_model
from os.path import join, dirname

ENCODING = 'latin1' # the encoding used by the mdb file
#~ ENCODING = 'utf8' 

GERMAN = []
GERMAN.append((17,1,u'ÖSHZ',u'Öffentliche Sozialhilfezentren'))


def objects():
  
    Sector = resolve_model('cbss.Sector')
  
    fn = join(dirname(__file__),'lijst_van_sectoren_liste_des_secteurs.csv')
    reader = ucsv.UnicodeReader(open(fn,'r'),encoding=ENCODING,delimiter=';')
    
    headers = reader.next()
    if headers != [u'Sector',u'',u'verkorte naam',u'Omschrijving',u'Abréviation',u'Nom']:
        raise Exception("Invalid file format: %r" % headers)
    reader.next() # ignore second header line
    code = None
    for row in reader:
        s0 = row[0].strip()
        s1 = row[1].strip()
        #~ if s0 == '17' and s1 == '0':
            #~ continue # 
        if s0 or s1:
            kw = {}
            if len(s0) > 0:
                #~ print repr(row[0])
                code = int(s0)
            kw.update(code=code)
            if row[1]:
                kw.update(subcode=int(row[1]))
            kw.update(**babel.babel_values('name',de=row[5],fr=row[5],nl=row[3]))
            kw.update(**babel.babel_values('abbr',de=row[4],fr=row[4],nl=row[2]))
            #~ print kw
            yield Sector(**kw)
        
    if 'de' in settings.LINO.languages:
        for code,subcode,abbr,name in GERMAN:
            sect = Sector.objects.get(code=code,subcode=subcode)
            if babel.DEFAULT_LANGUAGE == 'de':
                sect.abbr = abbr
                sect.name = name
            else :
                sect.abbr_de = abbr
                sect.name_de = name
            sect.save()

    # default value for SiteConfig.sector is "CPAS"
    #~ settings.LINO.site_config.sector = Sector.objects.get(code=17,subcode=1)
    #~ settings.LINO.site_config.save()





# -*- coding: utf-8 -*-
## Copyright 2011-2012 Luc Saffre
## This file is part of the Lino project.
## Lino is free software; you can redistribute it and/or modify 
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## Lino is distributed in the hope that it will be useful, 
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with Lino; if not, see <http://www.gnu.org/licenses/>.

"""
This module contains tests that are run on a demo database
with the following fixtures:

  std all_countries few_cities all_languages props demo
  
To run only this test suite::

  python manage.py test pcsw.DemoTest
  
See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`

Functions named `test0*` do not modify any data.

"""

import logging
logger = logging.getLogger(__name__)

import pprint

from django.db.utils import IntegrityError
from django.conf import settings
from django.utils.encoding import force_unicode
from django.utils import translation
from django.core.exceptions import ValidationError


#~ from django.utils import unittest
#~ from django.test.client import Client
#from lino.igen import models
#from lino.modlib.contacts.models import Contact, Companies
#from lino.modlib.countries.models import Country

from lino.utils import i2d
from lino.utils.jsgen import py2js
from lino.utils import babel
from lino.core.modeltools import resolve_model
#Companies = resolve_model('contacts.Companies')
from lino.utils.test import TestCase


#~ Person = resolve_model('contacts.Person')
#~ Property = resolve_model('properties.Property')
#~ PersonProperty = resolve_model('properties.PersonProperty')

class DemoTest(TestCase):
    #~ fixtures = [ 'std','demo' ]
    fixtures = 'std few_countries few_cities few_languages props cbss demo demo_events'.split()
    #~ fixtures = 'std all_countries few_cities all_languages props demo'.split()
    never_build_site_cache = True
    
    #~ def setUp(self):
        #~ settings.LINO.never_build_site_cache = True
        #~ super(DemoTest,self).setUp()


class PseudoRequest:
    def __init__(self,name):
        self.user = settings.LINO.user_model.objects.get(username=name)

            
def test001(self):
    """
    See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`.
    """
    #~ from lino.apps.pcsw.models import Person
    self.assertEquals(settings.LINO.person_model.objects.count(), 78)
    
    p = Person.objects.get(pk=118)
    #~ self.assertEquals(unicode(p), "ARENS Annette (118)")
    #~ self.assertEquals(unicode(p), "AUSDEMWALD Alfons (118)")
    self.assertEquals(unicode(p), "COLLARD Charlotte (118)")
    
    
        
def test002(self):
    """
    Tests whether SoftSkillsByPerson works and whether it returns language-specific labels.
    Bug discovered :doc:`/blog/2011/0228`.
    See also :doc:`/blog/2011/0531`.
    See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`.
    """
    from lino.modlib.users.models import User
    u = User.objects.get(username='root')
    lang = u.language
    u.language = '' # HTTP_ACCEPT_LANGUAGE works only when User.language empty
    u.save()
    
    pk = 124
    url = '/api/cv/SoftSkillsByPerson?mt=22&mk=%d&fmt=json' % pk
    
    if 'en' in babel.AVAILABLE_LANGUAGES:
        response = self.client.get(url,REMOTE_USER='root',HTTP_ACCEPT_LANGUAGE='en')
        result = self.check_json_result(response,'count rows gc_choices disabled_actions title')
        self.assertEqual(result['title'],"Properties of EVERTZ Bernd (%d)" % pk)
        self.assertEqual(len(result['rows']),2)
        row = result['rows'][0]
        self.assertEqual(row[0],"Obedient")
        #~ self.assertEqual(row[1],7)
        self.assertEqual(row[2],"moderate")
        self.assertEqual(row[3],"2")
        
    if 'de' in babel.AVAILABLE_LANGUAGES:
        response = self.client.get(url,REMOTE_USER='root',HTTP_ACCEPT_LANGUAGE='de')
        result = self.check_json_result(response,'count rows gc_choices disabled_actions title')
        self.assertEqual(result['title'],"Eigenschaften von EVERTZ Bernd (%d)" % pk)
        self.assertEqual(len(result['rows']),2)
        row = result['rows'][0]
        self.assertEqual(row[0],"Gehorsam")
        #~ self.assertEqual(row[1],7)
        self.assertEqual(row[2],u"mittelmäßig")
        self.assertEqual(row[3],"2")
        
    #~ 20111111 babel.set_language(None) # switch back to default language for subsequent tests
    
    u.language = lang
    u.save()
    
    
    #~ tf('http://127.0.0.1:8000/api/properties/SoftSkillsByPerson?_dc=1298881440121&fmt=json&mt=22&mk=15',
        #~ """
        #~ { 
        #~ count: 3, 
        #~ rows: [ 
          #~ [ "Gehorsam", 7, "mittelm\u00e4\u00dfig", "2", null, 53, "Sozialkompetenzen", 2 ], 
          #~ [ "F\u00fchrungsf\u00e4higkeit", 8, "mittelm\u00e4\u00dfig", "2", null, 54, "Sozialkompetenzen", 2 ], 
          #~ [ null, null, null, null, null, null, "Sozialkompetenzen", 2 ] 
        #~ ], 
        #~ gc_choices: [  ], 
        #~ title: "~Eigenschaften pro Person Arens Annette (15)" 
        #~ }
        #~ """)
    


def test003(self):
    """
    Test whether the AJAX call issued for Detail of Annette Arens is correct.
    See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`.
    
    """
    cases = [
    #  [ id,         name, recno, first, prev, next, last ]
       [ 119,  "Charlier",     7,   115,  168,  117, 198  ],
       [ 117,     "Arens",     2,   115,  119,   118, 198  ],
    ]
    # 
    for case in cases:
        url = '/api/contacts/Persons/%s?fmt=json' % case[0]
        response = self.client.get(url,REMOTE_USER='root')
        result = self.check_json_result(response,'navinfo disable_delete data id title disabled_actions')
        # disabled because they depend on local database sorting configuration
        # re-enabled because demo fixtures no longer contain cyrillic chars
        self.assertEqual(result['data']['last_name'],case[1])
        self.assertEqual(result['navinfo']['recno'],case[2])
        self.assertEqual(result['navinfo']['first'],case[3]) 
        self.assertEqual(result['navinfo']['prev'],case[4]) 
        self.assertEqual(result['navinfo']['next'],case[5])
        self.assertEqual(result['navinfo']['last'],case[6])
            
            
def test004(self):
    """
    Test whether date fields are correctly parsed.
    """
    for value in ('01.03.2011','15.03.2011'):
        url = '/api/jobs/Contracts/1'
        #~ data =  'applies_from='+value+'&applies_until=17.05.2009&company=R-Cycle%20'
        #~ 'Sperrgutsortierzentrum&companyHidden=83&contact=Arens%20Andreas%20(1'
        #~ '4)%20(Gesch%C3%A4ftsf%C3%BChrer)&contactHidden=2&date_decided=&date_e'
        #~ 'nded=&date_issued=&delay_type=Tage&delay_typeHidden=D&delay_value=0&du'
        #~ 'ration=&ending=Vertragsbeendigung%20ausw%C3%A4hlen...&endingHidden=&lan'
        #~ 'guage=Deutsch&languageHidden=de&person=Altenberg%20Hans%20(16)&personHi'
        #~ 'dden=16&reminder_date=11.11.2010&reminder_text=demo%20reminder&type=Kon'
        #~ 'vention%20Art.60%C2%A77%20Sozial%C3%B6konomie&typeHidden=1&user=root&us'
        #~ 'erHidden=4&user_asd=Benutzer%20ausw%C3%A4hlen...&user_asdHidden='
        data =  'applies_from='+value
        
        response = self.request_PUT(url,data,REMOTE_USER='root')
        result = self.check_json_result(response,'message success data_record')
        self.assertEqual(result['success'],True)
        self.assertEqual(result['data_record']['data']['applies_from'],value)
        
        url = "/api/jobs/Contracts/1?fmt=json"
        response = self.client.get(url,REMOTE_USER='root')
        #~ print 20110723, response
        result = self.check_json_result(response,'navinfo disable_delete data id title disabled_actions')
        self.assertEqual(result['data']['applies_from'],value)

def test005(self):
    """
    Simplification of test04, used to write :doc:`/tickets/27`.
    See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`.
    """
    url ='/api/countries/Countries/BE'
    value = 'Belgienx'
    data = 'name=%s&nameHidden=Belgienx&fmt=json' % value
    response = self.request_PUT(url,data,REMOTE_USER='root')
    #~ response = self.client.put(url,data,content_type='application/x-www-form-urlencoded')
    result = self.check_json_result(response,'message success data_record')
    self.assertEqual(result['success'],True)
    self.assertEqual(result['data_record']['data']['name'],value)
    
    url ='/api/countries/Countries/BE?fmt=json'
    response = self.client.get(url,REMOTE_USER='root')
    result = self.check_json_result(response,'navinfo disable_delete data id title disabled_actions')
    self.assertEqual(result['data']['name'],value)


def test006(self):
    """
    Testing BabelValues.
    See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`.
    """
    from lino.utils import babel
    from lino.apps.pcsw.models import Person
    #~ from lino.apps.pcsw.models import Property, PersonProperty
    Property = settings.LINO.modules.properties.Property
    PersonProperty = settings.LINO.modules.properties.PersonProperty
    annette = Person.objects.get(pk=118)
    self.assertEquals(unicode(annette), "ARENS Annette (118)")
    
    p = Property.objects.get(id=2) # "Obedient"
    pp = PersonProperty.objects.filter(property=p)[0]
    if 'en' in babel.AVAILABLE_LANGUAGES:
        babel.set_language('en')
        self.assertEquals(unicode(p), u"Obedient")
        self.assertEquals(unicode(pp), u"not at all")

    if 'de' in babel.AVAILABLE_LANGUAGES:
        babel.set_language('de')
        self.assertEquals(unicode(p), u"Gehorsam")
        self.assertEquals(unicode(pp), u"gar nicht")
    
    if 'fr' in babel.AVAILABLE_LANGUAGES:
        babel.set_language('fr')
        self.assertEquals(unicode(p), u"Obéissant")
        self.assertEquals(unicode(pp), u"pas du tout")
    
    babel.set_language(None) # switch back to default language for subsequent tests
    

def test007(self):
    """
    Test the number of rows returned for certain queries
    """
    cases = [
      ['contacts/Companies', 24],
      ['contacts/Persons', 53],
      ['pcsw/MyPersons',19],
      ['contacts/AllPersons', 74],
      ['contacts/AllPartners', 105],
      ['courses/Courses', 4],
      ['courses/CourseProviders', 3],
      ['courses/CourseOffers', 4],
      ['countries/Countries', 6],
      ['notes/Notes', 106],
      ['isip/Contracts', 21],
      ['jobs/JobProviders', 4],
      ['jobs/Jobs', 9],
      ['jobs/Contracts', 21], 
      ['jobs/Candidatures', 35],
      ['jobs/Studies', 3],
      #~ ['cal/Events', (204,205)], # seems that sometimes 204 is the correct number (depending on demo_date)
      ['cal/Events', (219,220,221,222)], # exact number can vary depending on demo_date
      ['cal/Tasks', 46],
      ['cal/Priorities', 10],
      ['notes/MyNotes', 28],
      ['properties/PropGroups', 4],
      ['/api/cal/RemindersByUser?fmt=json&limit=30&start=0&mt=5&mk=103&',22],
    ]
    for case in cases:
        if "?" in case[0]:
            url = case[0]
        else:
            url = '/api/%s?fmt=json&limit=30&start=0' % case[0]
        #~ logger.info("20120103 %s",url)
        response = self.client.get(url,REMOTE_USER='root')
        result = self.check_json_result(response,'count rows gc_choices disabled_actions title')
        #~ if result['count'] != case[1]:
            #~ logger.warning("%s",pprint.pformat(result['rows']))
        num = case[1]
        if not isinstance(num,tuple):
            num = [num]
        if not result['count'] in num:
            self.fail(
                "%s got %s rows instead of %s" % (case[0],result['count'],num))

    cases = [
      ['cv/SkillsByPerson/property',6],
    ]
    for case in cases:
        url = '/choices/%s?fmt=json&limit=10&start=0' % case[0]
        response = self.client.get(url,REMOTE_USER='root')
        result = self.check_json_result(response,'count rows')
        #~ if result['count'] != case[1]:
            #~ logger.warning("%s",pprint.pformat(result['rows']))
        self.assertEqual(result['count'],case[1],
            "%s got %d rows instead of %d" % (case[0],result['count'],case[1]))


def test009(self):
    """
    This tests for the bug discovered :doc:`/blog/2011/0610`.
    See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`.
    """
    #~ babel.set_language('en')
    url = '/choices/jobs/StudiesByPerson/city?start=0&limit=30&country=&query='
    response = self.client.get(url,REMOTE_USER='root')
    result = self.check_json_result(response,'count rows')
    #~ self.assertEqual(result['title'],u"Choices for city")
    self.assertEqual(len(result['rows']),30)
    #~ babel.set_language(None) # switch back to default language for subsequent tests

def test010(self):
    """
    Test the unique_together validation of City
    See :doc:`/blog/2011/0610` and :doc:`/blog/2011/0611`.
    See the source code at :srcref:`/lino/apps/pcsw/tests/pcsw_demo_tests.py`.
    """
    from lino.modlib.countries.models import City, Country
    be = Country.objects.get(pk='BE')
    try:
        City(name="Eupen",country=be,zip_code='4700').save()
    except IntegrityError:
        if settings.LINO.allow_duplicate_cities:
            self.fail("Got IntegrityError though allow_duplicate_cities should be allowed.")
    else:
        if not settings.LINO.allow_duplicate_cities:
            self.fail("Expected IntegrityError")
        
    
    try:
        be.city_set.create(name="Eupen",zip_code='4700')
    except IntegrityError:
        if settings.LINO.allow_duplicate_cities:
            self.fail("Got IntegrityError though allow_duplicate_cities should be allowed.")
    else:
        if not settings.LINO.allow_duplicate_cities:
            self.fail("Expected IntegrityError")
        
    
def test011(self):
    """
    Tests whether the user problem 
    described in :doc:`/blog/2011/1206` 
    is solved.
    """
    from lino.modlib.jobs.models import Contract
    obj = Contract.objects.get(pk=5)
    translation.activate('de')
    self.assertEqual(obj.contact.person.get_full_name(),"Herrn Hans ALTENBERG")
    #~ babel.set_language(None)
    translation.deactivate()
    
    
def test012(self):
    """
    Test whether the contact person of a jobs contract is correctly filled in
    when the provider has exactly one contact person.
    """
    from lino.modlib.jobs.models import Contract, JobProvider, Job
    from lino.apps.pcsw.models import Person
    from lino.modlib.users.models import User
    u = User.objects.get(username='root')
    #~ qs = Person.objects.order_by('last_name','first_name')
    p = Person.objects.get(pk=177) # Emil Eierschal
    #~ e = Employer.objects.get(pk=185)
    j = Job.objects.get(pk=1) # bisa
    c = Contract(person=p,user=u,job=j,applies_from=p.coached_from,duration=312)
    c.full_clean()
    c.save()
    self.assertEqual(c.contact.person.pk,118)
    #~ self.assertEqual(c.applies_until,p.coached_from+datetime.timedelta(days=))
    

def test014(self):
    """
    Tests for the bug discovered :doc:`/blog/2011/1222`.
    """
    for url in """\
    /choices/isip/Contract/person?start=0&limit=10&query=
    /choices/contacts/Person/city?start=0&limit=10&country=BE&query=
    /choices/jobs/Contract/duration
    """.splitlines():
      url = url.strip()
      if url and not url.startswith("#"):
          response = self.client.get(url,REMOTE_USER='root')
          result = self.check_json_result(response,'count rows')
          #~ self.assertEqual(result['title'],u"Choices for city")
          self.assertEqual(len(result['rows']),min(result['count'],10))

def test015(self):
    """
    Temporary bug on :doc:`/blog/2011/1223`.
    """
    url = '/api/contacts/Persons/-99999?fmt=json&an=insert'
    response = self.client.get(url,REMOTE_USER='root')
    result = self.check_json_result(response,'data phantom title')
    self.assertEqual(result['phantom'],True)

def test015b(self):
    """
    Test whether PropsByGroup has a detail.
    20120218 : "properties.PropsByGroup has no action u'detail'"
    """
    cases = [
      ('/api/properties/PropsByGroup/%s?mt=11&mk=1&an=detail&fmt=json',8),
      ('/api/contacts/AllPartners/%s?an=detail&fmt=json',117),
    ]
    for case in cases:
        url = case[0] % case[1]
        response = self.client.get(url,REMOTE_USER='root')
        result = self.check_json_result(response,
          'navinfo disable_delete data title disabled_actions id')
        self.assertEqual(result['id'],case[1])


def test016(self):
    """
    All rows of persons_by_user now clickable.
    See :doc:`/blog/2011/1223`.
    """
    cases = [
      ['root', 19],
      ['alicia', 17],
    ]
    for case in cases:
        url = '/api/pcsw/MyPersons?fmt=json&limit=30&start=0&su=%s' % case[0]
        response = self.client.get(url,REMOTE_USER='root')
        result = self.check_json_result(response,'count rows gc_choices disabled_actions title')
        self.assertEqual(result['count'],case[1])
        
        
def test017(self):
    from decimal import Decimal as D
    Budget = resolve_model('debts.Budget')
    bud = Budget.objects.get(pk=3)
    cases = [
      [ bud.msum('amount','I'), D('0') ],  #~ Monatliche Einkünfte
      [ bud.msum('amount','E'), D('0') ],  #~ Monatliche Ausgaben
      [ bud.ysum('amount','E')/12, D('0') ],   #~ Monatliche Reserve für jährliche Ausgaben
      [ bud.sum('monthly_rate','L'), D('0') ], #~ Raten der laufenden Kredite
      [ bud.sum('amount','L'), D('0.00')],  #Total Kredite / Schulden 
      [ bud.ysum('amount','E'), D('0.00')],  # Jährliche Ausgaben 
    ]
    
    for i,case in enumerate(cases):
        found,expected = case
        if found != expected:
            self.fail("Case %i : expected %s but found %s" % (i,expected,found))

    

        
        
def test101(self):
    """
    First we try to uncheck the is_jobprovider checkbox on 
    the Company view of a JobProvider. 
    This should fail since the JP has Jobs and Contracts.
    """
    Company = resolve_model('contacts.Company')
    JobProvider = resolve_model('jobs.JobProvider')
    Job = resolve_model('jobs.Job')
    Contract = resolve_model('jobs.Contract')
    bisaProvider = JobProvider.objects.get(pk=185)
    bisaCompany = Company.objects.get(pk=185)
    
    # it should work even on an imported partner
    save_iip = settings.LINO.is_imported_partner
    def f(obj): return True
    settings.LINO.is_imported_partner = f
    
    JOBS = Job.objects.filter(provider=bisaProvider)
    self.assertEqual(JOBS.count(),3)
    rr = PseudoRequest('root')
    try:
        Company.is_jobprovider.set_value_in_object(rr,bisaCompany,False)
        self.fail("Expected ValidationError")
    except ValidationError, e:
        # cannot delete because there are 3 Jobs referring to BISA
        pass
    for job in JOBS:
        for cont in Contract.objects.filter(job=job):
            cont.delete()
        job.delete()
    Company.is_jobprovider.set_value_in_object(rr,bisaCompany,False)
    
    bisaCompany = Company.objects.get(pk=185) # still exists
    
    try:
        bisaProvider = JobProvider.objects.get(pk=185)
        self.fail("Expected JobProvider.DoesNotExist")
    except JobProvider.DoesNotExist,e:
        pass
    
    # restore is_imported_partner method
    settings.LINO.is_imported_partner = save_iip
    